// 
//==============================================
// ETK(Embedded Training Kit) - AT91SAM7SE512
//==============================================
//
// File name : etk.h
// Project name : ETK-uCOSii
// Description : ETK-AT91SAM7SE512 شϴ 
// By : Roh, YoungSup(ysroh@suv.ac.kr)
//      Dept. of Embedded Systems. 
//      SUV ( Seoul University of Venture and Information )
// Date : 2007.12.14
// Modifications :
//    Rev 1.1 UART0    ߰(By JI(); ji77k7@hanmail.net) 
//
//===============================================
//


/*
bsp_etk.c ߰ؾ  
1. 7-segment ==> O.K.
2. Key Matrix
3. LCD interface
4. ADC 
5. IR receiver
6. Serial Port ==> O.K.   Rev 1.1
7. USB interface
8. LED ==> O.K.
9. External SRAM ==> O.K.
*/

/*
*********************************************************************************************************
*
*                                      APPLICATION CONFIGURATION
*
*                                            ETK-AT91SAM7SE512
*
* Filename      : etk.h
* Version       : V1.00
* Programmer(s) : ysroh
*********************************************************************************************************
*/

#ifndef __ETK_H__
#define __ETK_H__


//********************************************************************************************************
//
//		Global Functions for ETK-AT91SAM7SE512
// 
//********************************************************************************************************
#ifndef __ASSEMBLY__

#include <lib_def.h>
#include <cpu.h>
void Ext_SRAM_Init(void);
void Ext_SRAM_Clean(void);
void  ETK_Init (void);
CPU_INT32U  BSP_CPU_ClkFreq1 (void);
void SevenSeg_Init(void);
//INT8U Conv_To_7seg(INT8U c);
void SevenSeg_Putui(INT8U c);

// ++ ji(Rev 1.1) START
void Serial_Init();																						// UART0 ʱȭ
void Serial_dis();																						// ͷƮ Disable
void Serial_WrByte (CPU_INT08U tx_byte);											//   ۽ 
void Serial_WrStr (CPU_CHAR *tx_str);													// ڿ ۽
void Serial_Printf (CPU_CHAR *format, ...);										// UART0 Tx ۽
CPU_INT08U Serial_RdByte (void);															//   (Rx ͷƮ  )
void  Serial_RdStr (CPU_CHAR  *rx_str,CPU_INT32U   len);			// ڿ 
void irq_request ( CPU_INT32U irq, CPU_INT32U type, void (*irq_func ) (void ) );	// ͷƮ  Լ


#define RX_QUEUE_SIZE   256																		// Rx ͷƮ  Ǵ ť 
#define RX_QUEUE_MASK    0xFF																	// ȯ ť ũ 
CPU_CHAR     rxq_buff[RX_QUEUE_SIZE];													// ȯ ť 
CPU_INT32U   rxq_head;																				// ȯ ť  
CPU_INT32U   rxq_tail;																				// ȯ ť  
// ++ ji(Rev 1.1) END
// ++ ji(Rev 1.1)
void PWM_Init();//PWM ʱȭ  

#endif


//********************************************************************************************************
//
//		Flags
//
//********************************************************************************************************

#ifndef  DEBUG
#define  DEBUG    1
#endif

#define YES 1
#define NO 0


//********************************************************************************************************
//
//		For Seven Segment
//
//********************************************************************************************************

//     a
//   0000
// f 0  0 b
//   0000<--g
// e 0  0 c
//   0000 0<--Dp
//     d
//	0 : on
// 	1 : off
//                         abcDpdefg
//                         7654 3210
#define	_7SEG_0		0x11 //0001 0001
#define	_7SEG_1		0x9f //1001 1111
#define	_7SEG_2		0x32 //0011 0010
#define	_7SEG_3		0x16 //0001 0110
#define	_7SEG_4		0x9c //1001 1100
#define	_7SEG_5		0x54 //0101 0100
#define	_7SEG_6		0x50 //0101 0000
#define	_7SEG_7		0x1d //0001 1101
#define	_7SEG_8		0x10 //0001 0000
#define	_7SEG_9		0x14 //0001 0100
#define	_7SEG_A 	0x18 //0001 1000
#define	_7SEG_B 	0xd0 //1101 0000
#define	_7SEG_C 	0x71 //0111 0001
#define	_7SEG_D 	0x92 //1001 0010
#define	_7SEG_E 	0x70 //0111 0000
#define	_7SEG_F 	0x78 //0111 1000


/*
void SevenSeg_Init(void);
INT8U Conv_To_7seg(INT8U c);
INT8U SevenSeg_Printf(INT8U c);
*/

//********************************************************************************************************
//
//		Memory map of ETK-AT91SAM7SE512
//
//********************************************************************************************************

// Internal Flash Memory : 512k Bytes
#define FLASH_ADDR_START		0x00000000
#define FLASH_ADDR_END			0x00080000

// Internal SRAM Memory : 32k Bytes
#define SRAM_ADDR_START			0x00200000
#define SRAM_ADDR_END			0x00208000

// External SRAM Memory : 512k Bytes
#define EXT_SRAM_ADDR_START		0x10000000
#define EXT_SRAM_ADDR_END		0x10080000


//********************************************************************************************************
//
//		Definitions of Processor mode
//
//********************************************************************************************************

// Mode, correspords to bits 0-5 in CPSR of ARM core
#define MODE_BITS	0x1F		// Bit mask for mode bits in CPSR
#define USR_MODE	0x10		// User mode
#define FIQ_MODE	0x11		// Fast Interrupt Request mode
#define IRQ_MODE	0x12		// Interrupt Request mode
#define SVC_MODE	0x13		// Supervisor mode
#define ABT_MODE	0x17		// Abort mode
#define UND_MODE	0x1B		// Undefined Instruction mode
#define SYS_MODE	0x1F		// System mode

//********************************************************************************************************
//
//		Stack sizes of Processor mode
//
//********************************************************************************************************

// Location of stack
#define STACK_END_ADDR			SRAM_ADDR_END		// End of internal SRAM
//#define STACK_END_ADDR			EXT_SRAM_ADDR_END	// End of External SRAM

// Default size of stack
#define STACK_SIZE	0x100           // stack size = 256 bytes = 64 x 4 bytes

// Stack size of processor modes
#define  FIQ_STACK  STACK_END_ADDR - 4 - STACK_SIZE * 0		// by 1
#define  IRQ_STACK  STACK_END_ADDR - 4 - STACK_SIZE * 1		// by 2
#define  SVC_STACK  STACK_END_ADDR - 4 - STACK_SIZE * 3		// by 2
#define  UND_STACK  STACK_END_ADDR - 4 - STACK_SIZE * 5		// by 1
#define  ABT_STACK  STACK_END_ADDR - 4 - STACK_SIZE * 6		// by 1
#define  SYS_STACK  STACK_END_ADDR - 4 - STACK_SIZE * 7		// by 1
#define  END_STACK  STACK_END_ADDR -     STACK_SIZE * 10	// by 3


//********************************************************************************************************
//
//		Intial Values for ETK-AT91SAM7SE512 Board
//
//********************************************************************************************************

#define SMC_CSR_NWS_ETK				(0x02 	<<  0)		// Number of wait states, 3 wait states, 1 ~ 128 wait states
#define SMC_CSR_WSEN_ETK			(0x01	<<	7)		// Wait state enable - 1 wait state, 0 wait state if disable
#define SMC_CSR_TDF_ETK				(0x00	<<	8)		// Data float time, 1 cycle float, 0 ~ 15 cycles
#define SMC_CSR_BAT_ETK				(0x01	<<	12)		// Byte access type, 0 for two 8-bit wide device, 1 for a 16-bit wide device
#define SMC_CSR_DBW_ETK				(0x01	<<	13)		// Data bus width, 01b for 16-bit, 10b for 8-bit
#define SMC_CSR_DRP_ETK				(0x00	<<	15)		// Data read protocol, 0 for Standard, 1 for Early read protocol

#define SMC_CSR_ACSS_ETK			(0x01	<<	16)		// Address to chip select setup
#define SMC_CSR_RWSETUP_ETK			(0x01	<<	24)		// Read and Write signal setup time
#define SMC_CSR_RWHOLD_ETK			(0x01	<<	28)		// Read and write signal hold time


//********************************************************************************************************
//
//		Definitions for ETK-AT91SAM7SE512 Board
//
//********************************************************************************************************

// External Bus Interface : Address lines
#define  ETK_EBI_A00A17		(0x3ffff	<<	0)		// Address = A0 ~ A17
#define  ETK_EBI_A18A22		(   0x1f    << 16)      // Address = A18 ~ A22

// External Bus Interface : Data lines
#define EXT_EBI_D0D15		(0xffff 	<< 0)		// data = d0 ~ d15

//********************************************************************************************************
// PIO CONTROLLER A							// IO Line/Peripheral A/Peripheral B, ETK
//********************************************************************************************************

//LEDs
#define  ETK_LED0			DEF_BIT_00		// PA0/PWM0/A0, LED0
#define  ETK_LED1			DEF_BIT_01		// PA1/PWM1/A1, LED1
#define  ETK_LED2			DEF_BIT_02		// PA2/PWM2/A2, LED2

// PWMs
#define  ETK_PWM0			DEF_BIT_00		// PA0/PWM0/A0, PWM0
#define  ETK_PWM1			DEF_BIT_01		// PA1/PWM1/A1, PWM1
#define  ETK_PWM2			DEF_BIT_02		// PA2/PWM2/A2, PWM2

// EEPROM
#define  ETK_TWD			DEF_BIT_03		// PA3/TWD/A3,  TWD/SDA
#define  ETK_TWCK			DEF_BIT_04		// PA4/TWCK/A4, TWCK/SCL

// Serial Port
#define  ETK_RXD0			DEF_BIT_05		// PA5/RXD0/A5, RXD0
#define  ETK_TXD0			DEF_BIT_06		// PA6/TXD0/A6, TXD0
#define  ETK_RTS0			DEF_BIT_07		// PA7/RTS0/A7, RTS0
#define  ETK_CTS0			DEF_BIT_08		// PA8/CTS0/A8, CTS0

// Debug Serial Port
#define  ETK_DRXD			DEF_BIT_09		// PA9/DRXD/A9,   DRXD
#define  ETK_DTXD			DEF_BIT_10		// PA10/DTXD/A10, DTXD

// SPI Bus
#define  ETK_NPCS0			DEF_BIT_11		// PA11/NPCS0/A11, NPCS0
#define  ETK_MISO			DEF_BIT_12		// PA12/MISO/A12,  MISO
#define  ETK_MOSI			DEF_BIT_13		// PA13/MOSI/A13,  MOSI
#define  ETK_SPCK			DEF_BIT_14		// PA14/SPCK/A14,  SPCK

// Not Used
//#define  ETK_				DEF_BIT_15		// PA15/TF/A15, Not Used

// USB_DP Pull-Up
#define ETK_USB_DP_PU		DEF_BIT_16		// PA16/TK/A16, USB_DP_PU

// ADC Inputs
#define  ETK_AD0			DEF_BIT_17		// PA17/TD/A17,  AD0
#define  ETK_AD1			DEF_BIT_18		// PA18/TD/NBS3, AD1
#define  ETK_AD2			DEF_BIT_19		// PA19/TD/NCS4, AD2
#define  ETK_AD3			DEF_BIT_20		// PA20/TD/NCS2, AD3

// Not Used
//#define  ETK_				DEF_BIT_21		// PA21/RXD1/NCS6, Not Used
//#define  ETK_				DEF_BIT_22		// PA22/TXD1/NCS5, Not Used

// External Bus Interface : Byte Select 1 - Upper byte select
#define  ETK_NBS1			DEF_BIT_23		// PA23/SCK1/NBS1, NBS1

// Key pad interface
#define  ETK_RTN0			DEF_BIT_24		// PA24/RTS1/SDA10,     RTN0
#define  ETK_RTN1			DEF_BIT_25		// PA25/CTS1/SDCKE,     RTN1
#define  ETK_RTN2			DEF_BIT_26		// PA26/DCD1/NCS1_SDCS, RTN2
#define  ETK_RTN3			DEF_BIT_27		// PA27/DTR1/SDWE,      RTN3
#define  ETK_SCAN0			DEF_BIT_28		// PA28/DSR1/CAS,       SCAN0
#define  ETK_SCAN1			DEF_BIT_29		// PA29/RI1/RAS,        SCAN1
#define  ETK_SCAN2			DEF_BIT_30		// PA30/IRQ1/D30,       SCAN2
#define  ETK_SCAN3			DEF_BIT_31		// PA31/NPCS1/D31,      SCAN3

 
//********************************************************************************************************
// PIO CONTROLLER B							// IO Line/Peripheral A/Peripheral B, ETK
//********************************************************************************************************

// External Bus Interface : Address
#define  ETK_A0				DEF_BIT_00		// PB0/TIOA0/A0,   A0, Lower byte select
#define  ETK_A1				DEF_BIT_01		// PB1/TIOB0/A1,   A1
#define  ETK_A2				DEF_BIT_02		// PB2/SCK0/A2,    A2
#define  ETK_A3				DEF_BIT_03		// PB3/NPCS3/A3,   A3
#define  ETK_A4				DEF_BIT_04		// PB4/TCLK0/A4,   A4
#define  ETK_A5				DEF_BIT_05		// PB5/NPCS3/A5,   A5
#define  ETK_A6				DEF_BIT_06		// PB6/PCK0/A6,    A6
#define  ETK_A7				DEF_BIT_07		// PB7/PWM3/A7,    A7

#define  ETK_A8				DEF_BIT_08		// PB8/ADTRG/A8,   A8
#define  ETK_A9				DEF_BIT_09		// PB9/NPCS1/A9,   A9
#define  ETK_A10			DEF_BIT_10		// PB10/NPCS2/A10, A10
#define  ETK_A11			DEF_BIT_11		// PB11/PWM0/A11,  A11
#define  ETK_A12			DEF_BIT_12		// PB12/PWM1/A12,  A12
#define  ETK_A13			DEF_BIT_13		// PB13/PWM2/A13,  A13
#define  ETK_A14			DEF_BIT_14		// PB14/PWM3/A14,  A14
#define  ETK_A15			DEF_BIT_15		// PB15/TIOA1/A15, A15

#define  ETK_A16			DEF_BIT_16		// PB16/TIOB1/A16, A16
#define  ETK_A17			DEF_BIT_17		// PB17/PCK1/A17,  A17

// Not Used
//#define  ETK_				DEF_BIT_18		// PB18/PCK2/D16,  Not Used

// External Fast Interrupt request
#define  ETK_FIQ			DEF_BIT_19		// PB19/FIQ/D17, FIQ

// External Interrupt request
#define  ETK_IRQ			DEF_BIT_20		// PB20/IRQ0/D18, IRQ

// 7(Seven) Segment
#define  ETK_7SEG_LATCH		DEF_BIT_21		// PB21/PCK1/D19,  7SEG_LATCH
#define  ETK_SHIFT_CLOCK	DEF_BIT_22		// PB22/NPCS3/D20, SHIFT_CLOCK
#define  ETK_SHIFT_DATA		DEF_BIT_23		// PB23/PWM0/D21,  SHIFT_DATA

// Character LCD Interface
#define  ETK_LCD_RS			DEF_BIT_24		// PB24/PWM1/D22,   LCS_RS
#define  ETK_LCD_RW			DEF_BIT_25		// PB25/PWM2/D23,   LCD_RW

// InfraRed Interface : Remocon receiver
#define  ETK_IR				DEF_BIT_26		// PB26/TIOA2/D24,  IR

// Character LCD Interface
#define  ETK_LCD_EN			DEF_BIT_27		// PB27/TIOB2/D25,  LCD_EN
#define  ETK_LCD_D4			DEF_BIT_28		// PB28/TCLK1/D26,  LCD_D4
#define  ETK_LCD_D5			DEF_BIT_29		// PB29/TCLK2/D27,  LCD_D5
#define  ETK_LCD_D6			DEF_BIT_30		// PB30/NPCS3/D28,  LCD_D6
#define  ETK_LCD_D7			DEF_BIT_31		// PB31/PCK2/D29,   LCD_D7

//********************************************************************************************************
// PIO CONTROLLER C							// IO Line/Peripheral A/Peripheral B, ETK
//********************************************************************************************************

// External Bus Interface : Data Bus
#define  ETK_D0				DEF_BIT_00		// PC0/D0/   ,              D0
#define  ETK_D1				DEF_BIT_01		// PC1/D1/   ,              D1
#define  ETK_D2				DEF_BIT_02		// PC2/D2/   ,              D2
#define  ETK_D3				DEF_BIT_03		// PC3/D3/   ,              D3
#define  ETK_D4				DEF_BIT_04		// PC4/D4/   ,              D4
#define  ETK_D5				DEF_BIT_05		// PC5/D5/   ,              D5
#define  ETK_D6				DEF_BIT_06		// PC6/D6/   ,              D6
#define  ETK_D7				DEF_BIT_07		// PC7/D7/   ,              D7

#define  ETK_D8				DEF_BIT_08		// PC8/D8/RTS1,             D8
#define  ETK_D9				DEF_BIT_09		// PC9/D9/DTR1,             D9
#define  ETK_D10			DEF_BIT_10		// PC10/D10/PCK0,           D10
#define  ETK_D11			DEF_BIT_11		// PC11/D11/PCK1,           D11
#define  ETK_D12			DEF_BIT_12		// PC12/D12/PCK2,           D12
#define  ETK_D13			DEF_BIT_13		// PC13/D13/   ,            D13
#define  ETK_D14			DEF_BIT_14		// PC14/D14/NPCS1,          D14
#define  ETK_D15			DEF_BIT_15		// PC15/D15/NCS3_NANDCS,    D15

// External Bus Interface : Address Bus
#define  ETK_A18			DEF_BIT_16		// PC16/A18/NWAIT,          A18
#define  ETK_A19			DEF_BIT_17		// PC17/A19/NANDOE,         A19
#define  ETK_A20			DEF_BIT_18		// PC18/A20/NANDWE,         A20
#define  ETK_A21			DEF_BIT_19		// PC19/A21/      ,         A21
#define  ETK_A22			DEF_BIT_20		// PC20/A22/NCS7,           A22

// External Bus Interface : 
#define  ETK_NWE			DEF_BIT_21		// PC21/     /NWE,			NWE
#define  ETK_NRD			DEF_BIT_22		// PC22/     /NRD,			NRD
#define  ETK_NCS0			DEF_BIT_23		// PC23/CFRNW/NCS0,			NCS0

#endif

