// 
//==============================================
// ETK(Embedded Training Kit) - AT91SAM7SE512
//==============================================
//
// File name : app.c
// Project name : ETK-uCOSii
// Description : Sample application program
// By : Roh, YoungSup(ysroh@suv.ac.kr)
//      Dept. of Embedded Systems. 
//      SUV ( Seoul University of Venture and Information )
// Date : 2007.12.14
// Modifications :
//     1. OS_PROBE ִ app.c  Ͽ.
//     2. Probe شǴ  ڵ .
//     3. AppFormatDec() Լ 
//           Ǵ  , ʿ ߰  .
//     4. AppTaskTst1() AppTaskTst2() ߰
//		 5.  ӿ Լ   ߰(2008.02.23 ̻ golsm21@naver.com)
//           	struct Node, 
//						int PrtPoint(void);
//						int InitCard(void);
//						int GetCard(int nUser);
//						int PrtCard(void);
//						int Judge(void);
//						CPU_CHAR BJ_getc (void); 
//===============================================
//


/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#include <includes.h>
#include <time.h>

/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*/

// ++LSM Black Jack ӿ start
#define KIND_CARD 4					// ī 
#define COUNT_CARD 13				// ī 
#define INIT_POINT 5				// ⺻ 
#define NOTUSED 0					// ʱ ī 
#define USED -1						// ī 
#define USEDCARD 3					//  ִ ī 
#define COMPUTER 1					// ǻ
#define PLAYER 2					// ÷̾

struct Node 
{
	int nCrdKnd[USEDCARD];
	int nCrdCnt[USEDCARD];
	int nPtCrd;
	int nPoint;
} Player, Computer;

int card[KIND_CARD][COUNT_CARD];	// ī [][]

/* Fuction Prototype */
int PrtPoint(void);
int InitCard(void);
int GetCard(int nUser);
int PrtCard(void);
int Judge(void);
CPU_CHAR BJ_getc (void);
// ++LSM Black Jack ӿ end

/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/

/* ----------------- APPLICATION GLOBALS ------------------ */
static  OS_STK        AppTaskStartStk[APP_TASK_START_STK_SIZE];

// ++ roh,  task
static  OS_STK        AppTaskTst1Stk[APP_TASK_TST1_STK_SIZE];
static  OS_STK        AppTaskTst2Stk[APP_TASK_TST2_STK_SIZE];

// ++ roh,  semaphore
OS_EVENT *sem_Ser_Printf;


/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static  void  AppTaskStart    (void        *p_arg);
static  void  AppTaskCreate   (void);

// ++ roh
static  void  AppTaskTst1     (void          *p_arg);
static  void  AppTaskTst2     (void          *p_arg);


/*
*********************************************************************************************************
*                                                main()
*
* Description : The standard entry point for C code.  It is assumed that your code will call
*               main() once you have performed all necessary initialization.
*
* Argument(s) : none.
*
* Return(s)   : none.
*********************************************************************************************************
*/

int  main (void)
{
    CPU_INT08U  err;


    BSP_IntDisAll();                                            /* Disable all interrupts until we are ready to accept them */
    
    OSInit();                                                   /* Initialize "uC/OS-II, The Real-Time Kernel"              */
   // irq_init(); 
    OSTaskCreateExt(AppTaskStart,                               /* Create the start task                                    */
                    (void *)0,
                    (OS_STK *)&AppTaskStartStk[APP_TASK_START_STK_SIZE - 1],
                    APP_TASK_START_PRIO,
                    APP_TASK_START_PRIO,
                    (OS_STK *)&AppTaskStartStk[0],
                    APP_TASK_START_STK_SIZE,
                    (void *)0,
                    OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR);

#if (OS_TASK_NAME_SIZE > 13)
	OSTaskNameSet(APP_TASK_START_PRIO, (INT8U *)"Start Task", &err);
#endif

    // Create semaphore for the debug serial port
    sem_Ser_Printf = OSSemCreate(1);

    OSStart();                                                  /* Start multitasking (i.e. give control to uC/OS-II)       */

	return 0;
}


/*
*********************************************************************************************************
*                                          AppTaskStart()
*
* Description : The startup task.  The uC/OS-II ticker should only be initialize once multitasking starts.
*
* Argument(s) : p_arg       Argument passed to 'AppTaskStart()' by 'OSTaskCreate()'.
*
* Return(s)   : none.
*
* Note(s)     : (1) The first line of code is used to prevent a compiler warning because 'p_arg' is not
*                   used.  The compiler should not generate any code for this statement.
*
*               (2) Interrupts are enabled once the task starts because the I-bit of the CCR register was
*                   set to 0 by 'OSTaskCreate()'.
*********************************************************************************************************
*/

static  void  AppTaskStart (void *p_arg)
{
	INT8U sem_err;

    (void)p_arg;

    BSP_Init();
                                    /* Initialize BSP functions                                 */

#if (OS_TASK_STAT_EN > 0)
    OSStatInit();                   /* Determine CPU capacity                                   */
#endif

     
		Serial_Init();									/* Init. the Serial Channel                           */
		Ser_Init();											/* Init. the Debug Serial Channel                                 */

    //Serial_dis();
  
    OSSemPend(sem_Ser_Printf, 0, &sem_err);
    Serial_Printf((CPU_CHAR *)"ETK : INIT : Serial Channel\n\r");
    Ser_Printf((CPU_CHAR *)"ETK : INIT : Debug Serial Channel\n\r");
    OSSemPost(sem_Ser_Printf);


	// Creates application task
    AppTaskCreate();
	
	/* Task body, always written as an infinite loop.           */
    while (DEF_TRUE) { 
	    OSTimeDlyHMSM(0, 0, 10, 0);
    }
}


/*
*********************************************************************************************************
*                                      AppTaskCreate()
*
* Description :  Create the application tasks.
*
* Argument(s) :  none.
*
* Return(s)   :  none.
*********************************************************************************************************
*/

static  void  AppTaskCreate (void)
{
    CPU_INT08U  err;


	// Create TaskTst1
    OSTaskCreateExt(AppTaskTst1,
                    (void *)0,
                    (OS_STK *)&AppTaskTst1Stk[APP_TASK_TST1_STK_SIZE - 1],
                    APP_TASK_TST1_PRIO,
                    APP_TASK_TST1_PRIO,
                    (OS_STK *)&AppTaskTst1Stk[0],
                    APP_TASK_TST1_STK_SIZE,
                    (void *)0,
                    OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR);

#if (OS_TASK_NAME_SIZE > 8)
    OSTaskNameSet(APP_TASK_TST1_PRIO, (INT8U *)"Test1", &err);
#endif

	// Create TaskTst2
    OSTaskCreateExt(AppTaskTst2,
                    (void *)0,
                    (OS_STK *)&AppTaskTst2Stk[APP_TASK_TST2_STK_SIZE - 1],
                    APP_TASK_TST2_PRIO,
                    APP_TASK_TST2_PRIO,
                    (OS_STK *)&AppTaskTst2Stk[0],
                    APP_TASK_TST2_STK_SIZE,
                    (void *)0,
                    OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR);

#if (OS_TASK_NAME_SIZE > 8)
    OSTaskNameSet(APP_TASK_TST2_PRIO, (INT8U *)"Test2", &err);
#endif


}

/*
*********************************************************************************************************
*                                    AppTaskTst1() and AppTaskTst2()
*
* Description : for test.
*
* Argument(s) : p_arg       Argument passed to 'AppTaskTst1' and AppTaskTst2' by 'OSTaskCreate()'.
*
* Return(s)   : none.
*
* Note(s)     : (1) The first line of code is used to prevent a compiler warning because 'p_arg' is not
*                   used.  The compiler should not generate any code for this statement.
*********************************************************************************************************
*/
/*
*********************************************************************************************************
*									 AppTaskTst1() 
*
* Description : for Basic main Task
*
* Argument(s) : p_arg		Argument passed to 'AppTaskTst1' and AppTaskTst2' by 'OSTaskCreate()'.
*
* Return(s)   : none.
*
* Note(s)	  : (1) The first line of code is used to prevent a compiler warning because 'p_arg' is not
*			   used.  The compiler should not generate any code for this statement.
*             (2) Black Jack main Task                 
*********************************************************************************************************
*/

// ++ roh
static  void  AppTaskTst1 (void *p_arg)
{
    INT8U sem_err;
    INT8U  i = 0;

    (void)p_arg;
	
		// Task body, always written as an infinite loop.
		while (DEF_TRUE) {
      OSSemPend(sem_Ser_Printf, 0, &sem_err);
  		Ser_Printf((CPU_CHAR *)"ETK : AppTaskTst1\r\n");
      OSSemPost(sem_Ser_Printf);

      LED_Toggle((CPU_INT08U) 3);			// toggle the LED0
			SevenSeg_Putui(i++);

      OSTimeDlyHMSM(0, 0, 1, 0);		// one second
    }
}

static  void  AppTaskTst2 (void *p_arg)
{
    INT8U sem_err;

    (void)p_arg;

		// Task body, always written as an infinite loop.
    while (DEF_TRUE) {
      OSSemPend(sem_Ser_Printf, 0, &sem_err);
  		Ser_Printf((CPU_CHAR *)"ETK : AppTaskTst2\r\n");
      OSSemPost(sem_Ser_Printf);
      LED_Toggle((CPU_INT08U) 1);			// toggle the LED1

      OSTimeDlyHMSM(0, 0, 2, 0);		// two second
    }
}
// ++ roh


/*
START****************************************************************************************************
*********************************************************************************************************
**                                     α׷  START
**
**		   α׷  Դϴ. ׽Ʈ  ø̼ Ǵ Դϴ.
*********************************************************************************************************
****************************************************************************************************START
*/

/*
// UART ͷƮ ׽Ʈ START
static  void  AppTaskTst1 (void *p_arg)
{
  INT8U sem_err;

  (void)p_arg;
	
	Serial_Printf((CPU_CHAR *)"\t******************************\n\r");
	Serial_Printf((CPU_CHAR *)"\t*     Serial Test Program    *\n\r");
	Serial_Printf((CPU_CHAR *)"\t******************************\n\r");
	Serial_Printf((CPU_CHAR *)"\n\r");
	Serial_Printf((CPU_CHAR *)"Էµ ڰ ȭ鿡 µǴ α׷ Դϴ.\n\r");
	
	// Task body, always written as an infinite loop.
	while (DEF_TRUE) {
	    OSSemPend(sem_Ser_Printf, 0, &sem_err);
  		Serial_RdStr(rxq_buff,rxq_head);							// ť ۿ   ٽ 
  		rxq_head = 0;																	//  Ϸȵ  ʱȭ
      OSSemPost(sem_Ser_Printf);
      
	    OSTimeDlyHMSM(0, 0, 0, 100);										// one second
    }
}

static  void  AppTaskTst2 (void *p_arg)
{
    INT8U sem_err;
    
    (void)p_arg;

		// Task body, always written as an infinite loop.
    while (DEF_TRUE) {
      LED_Toggle((CPU_INT08U) 1);			// toggle the LED1
      
     	OSTimeDlyHMSM(0, 0, 0, 100);		// two second
    }
}
// UART ͷƮ ׽Ʈ END
*/


/*
// PWM ׽Ʈ START
static	void  AppTaskTst1 (void *p_arg)
{
	INT8U sem_err;
	INT8U  i = 0;
	
	static int co = 100; 
	(void)p_arg;

	Serial_Printf((CPU_CHAR *)"\t******************************\n\r");
	Serial_Printf((CPU_CHAR *)"\t*       PWM Test Program     *\n\r");
	Serial_Printf((CPU_CHAR *)"\t******************************\n\r");
	Serial_Printf((CPU_CHAR *)"\n\r");
	Serial_Printf((CPU_CHAR *)"LED  Ⱑ   ϴ.\n\r");
	
	
	// Task body, always written as an infinite loop.
	while (DEF_TRUE) {
		OSSemPend(sem_Ser_Printf, 0, &sem_err);
		AT91C_BASE_PWMC_CH2->PWMC_CDTYR =AT91C_BASE_PWMC_CH2->PWMC_CPRDR/co;
		co = co-10;
		if(co==0) {
			co = 100;
			Serial_Printf((CPU_CHAR *)"Ϸ.\n\r");
		}
		OSSemPost(sem_Ser_Printf);
		OSTimeDlyHMSM(0, 0, 1, 0);		// one second
		}
}

static  void  AppTaskTst2 (void *p_arg)
{
    INT8U sem_err;
    INT8U  i = 0;
    
    (void)p_arg;

		// Task body, always written as an infinite loop.
    while (DEF_TRUE) {
      
     	OSTimeDlyHMSM(0, 0, 0, 500);		// two second
    }
}
// PWM ׽Ʈ END
*/


/*
//   START
static	void  AppTaskTst1 (void *p_arg)
{
	CPU_CHAR chAns;												//  亯 (Y/N)

	Computer.nPoint = INIT_POINT;
	Player.nPoint = INIT_POINT;
	

	Serial_Printf((CPU_CHAR *)"******************************************\n\r");
	Serial_Printf((CPU_CHAR *)"*              Black Jack                *\n\r");
	Serial_Printf((CPU_CHAR *)"******************************************\n\r");

	// Task body, always written as an infinite loop.
	while (DEF_TRUE) {
		//   ǥ
		PrtPoint();

		// ī带 ʱȭ
		InitCard();

		// ǻͰ ī带  ̴´
		GetCard(COMPUTER);

		// ÷̾ ī带  
		GetCard(PLAYER);
		GetCard(PLAYER);

		// ī带 ǥѴ
		PrtCard();

		// ÷̾     Ѵ
		Serial_Printf((CPU_CHAR *)"Do You Want A Card? (y/n) :");
		chAns = BJ_getc();
		Serial_Printf((CPU_CHAR *)"%c", chAns);
		if ( chAns == (CPU_CHAR)'y' || chAns == (CPU_CHAR)'Y' ){
			GetCard(PLAYER);
		} // if getchar END

		// ǻͰ ī带   ޴´
		GetCard(COMPUTER);

		// ǻͰ     Ѵ
		if ( Computer.nCrdCnt[0] + Computer.nCrdCnt[1] <= 16){
			GetCard(COMPUTER);
			Serial_Printf((CPU_CHAR *)"Computer get a card..\n\r");
		} // if computer_card END

		// ǻͿ ÷̾ ī带 ǥѴ
		PrtCard();

		//    Ѵ
		Judge();		
			
		OSTimeDlyHMSM(0, 0, 1, 0);		// one second
		}
}

static  void  AppTaskTst2 (void *p_arg)
{
    INT8U sem_err;

    (void)p_arg;

	// Task body, always written as an infinite loop.
    while (DEF_TRUE) {

              OSSemPend(sem_Ser_Printf, 0, &sem_err);
 	            SevenSeg_Putui(Player.nPoint);
			  			OSSemPost(sem_Ser_Printf);
              LED_Toggle((CPU_INT08U) 1);			// toggle the LED1

              OSTimeDlyHMSM(0, 0, 2, 0);		// two second
    }
}
//   END
*/

/*
END******************************************************************************************************
*********************************************************************************************************
**                                     α׷  END
*********************************************************************************************************
******************************************************************************************************END
*/




/*
*********************************************************************************************************
*********************************************************************************************************
**                                         uC/OS-II APP HOOKS
*********************************************************************************************************
*********************************************************************************************************
*/

#if (OS_APP_HOOKS_EN > 0)
/*
*********************************************************************************************************
*                                      TASK CREATION HOOK (APPLICATION)
*
* Description : This function is called when a task is created.
*
* Argument(s) : ptcb   is a pointer to the task control block of the task being created.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*********************************************************************************************************
*/

void  App_TaskCreateHook (OS_TCB *ptcb)
{


}

/*
*********************************************************************************************************
*                                    TASK DELETION HOOK (APPLICATION)
*
* Description : This function is called when a task is deleted.
*
* Argument(s) : ptcb   is a pointer to the task control block of the task being deleted.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*********************************************************************************************************
*/

void  App_TaskDelHook (OS_TCB *ptcb)
{
    (void)ptcb;
}

/*
*********************************************************************************************************
*                                      IDLE TASK HOOK (APPLICATION)
*
* Description : This function is called by OSTaskIdleHook(), which is called by the idle task.  This hook
*               has been added to allow you to do such things as STOP the CPU to conserve power.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts are enabled during this call.
*********************************************************************************************************
*/

#if OS_VERSION >= 251
void  App_TaskIdleHook (void)
{
}
#endif

/*
*********************************************************************************************************
*                                        STATISTIC TASK HOOK (APPLICATION)
*
* Description : This function is called by OSTaskStatHook(), which is called every second by uC/OS-II's
*               statistics task.  This allows your application to add functionality to the statistics task.
*
* Argument(s) : none.
*********************************************************************************************************
*/

void  App_TaskStatHook (void)
{
}

/*
*********************************************************************************************************
*                                        TASK SWITCH HOOK (APPLICATION)
*
* Description : This function is called when a task switch is performed.  This allows you to perform other
*               operations during a context switch.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*
*               (2) It is assumed that the global pointer 'OSTCBHighRdy' points to the TCB of the task that
*                   will be 'switched in' (i.e. the highest priority task) and, 'OSTCBCur' points to the
*                  task being switched out (i.e. the preempted task).
*********************************************************************************************************
*/

#if OS_TASK_SW_HOOK_EN > 0
void  App_TaskSwHook (void)
{


}
#endif

/*
*********************************************************************************************************
*                                     OS_TCBInit() HOOK (APPLICATION)
*
* Description : This function is called by OSTCBInitHook(), which is called by OS_TCBInit() after setting
*               up most of the TCB.
*
* Argument(s) : ptcb    is a pointer to the TCB of the task being created.
*
* Note(s)     : (1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/

#if OS_VERSION >= 204
void  App_TCBInitHook (OS_TCB *ptcb)
{
    (void)ptcb;
}
#endif

/*
*********************************************************************************************************
*                                        TICK HOOK (APPLICATION)
*
* Description : This function is called every tick.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/

#if OS_TIME_TICK_HOOK_EN > 0
void  App_TimeTickHook (void)
{


}
#endif
#endif

/* ++LSM
*********************************************************************************************************
*********************************************************************************************************
**                                         Black Jack Լ
*********************************************************************************************************
*********************************************************************************************************
*/

/* ++LSM
*********************************************************************************************************
*                                        BJ_getc (void)
*
* Description : Black Jack  UART Rx ſ Լ
*
* Argument(s) : none.
*
* Note(s)     : (1)    Է½ñ ⸦ ϱ  Ⱑ ϵ Լ  
*********************************************************************************************************
*/
CPU_CHAR BJ_getc (void)
{
    CPU_CHAR data;
#if OS_CRITICAL_METHOD == 3                              
    OS_CPU_SR     cpu_sr = 0;
#endif
    
    do
    {
        OS_ENTER_CRITICAL ();        
        if (rxq_head != rxq_tail)
        {
            break;
        }
        OS_EXIT_CRITICAL();                
        OSTimeDly (2);
    }    
    while (DEF_TRUE);
    
    data = rxq_buff[rxq_tail++]; 
		rxq_head &= RX_QUEUE_MASK;
		
    OS_EXIT_CRITICAL();
                    
    return data;
}

/* ++LSM
*********************************************************************************************************
*                                        PrtPoint(void)
*
* Description :   ǥϴ Լ
*
* Argument(s) : none.
*
* Note(s)     : (1)
*********************************************************************************************************
*/
int PrtPoint(void)
{
	Serial_Printf((CPU_CHAR *)"\n\r");
	Serial_Printf((CPU_CHAR *)"+++ Computer : %d, Player : %d +++ \n\r", Computer.nPoint, Player.nPoint);
	Serial_Printf((CPU_CHAR *)"\n\r");
	return 0;
} // PrtPoint END


/* ++LSM
*********************************************************************************************************
*                                        InitCard(void)
*
* Description : ī带 ʱȭ ϴ Լ
*
* Argument(s) : none.
*
* Note(s)     : (1)
*********************************************************************************************************
*/
int InitCard(void)
{
	int i,j;		// ӽ 

	// ī ʱȭ
	for ( i = 0 ; i < KIND_CARD ; i++){
		for ( j = 0 ; j < COUNT_CARD ; j++){
			card[i][j] = NOTUSED;
		} // for j END
	} // for i END

	// Computer and Player ī ʱȭ
	for ( i = 0 ; i < USEDCARD ; i++ ){
		Computer.nCrdCnt[i] = NOTUSED;
		Player.nCrdCnt[i] = NOTUSED;
	} // for i END

	// Computer and Player ī  ʱȭ
	Computer.nPtCrd = 0;
	Player.nPtCrd = 0;

	return 0;
} // InitCard END


/* ++LSM
*********************************************************************************************************
*                                        GetCard(int nUser)
*
* Description : nUser   ǻ Ȥ ڿ ī带 ִ Լ
*
* Argument(s) : nUser ī带  
*
* Note(s)     : (1)
*********************************************************************************************************
*/
int GetCard(int nUser)
{
	int i, j;			// ӽ 

	do{
		i = rand() % 4;
		j = rand() % 13;
	} while( card[i][j] == USED );

	card[i][j] = USED;
	
	switch( nUser ){
	case COMPUTER:
		Computer.nCrdCnt[Computer.nPtCrd] = j + 1;
		Computer.nCrdKnd[Computer.nPtCrd] = i;
		Computer.nPtCrd++;
		break;
	case PLAYER:
		Player.nCrdCnt[Player.nPtCrd] = j + 1;
		Player.nCrdKnd[Player.nPtCrd] = i;
		Player.nPtCrd++;
	    break;
	default:
	    break;
	}

	return 0;
} // GetCard END

/* ++LSM
*********************************************************************************************************
*                                        PrtCard(void)
*
* Description : ī带 ǥϴ Լ
*
* Argument(s) : none.
*
* Note(s)     : (1)
*********************************************************************************************************
*/
int PrtCard(void)
{
	int i = 0;			// ӽ 

	Serial_Printf((CPU_CHAR *)"Computer Card :");
	while (Computer.nCrdCnt[i] != 0){

		// ī    (ڿ )
		switch(Computer.nCrdKnd[i]){
		case 0 :
			Serial_Printf((CPU_CHAR *)"[%s%2d]", (CPU_CHAR *)"", Computer.nCrdCnt[i]);
			break;
		case 1 :
			Serial_Printf((CPU_CHAR *)"[%s%2d]", (CPU_CHAR *)"", Computer.nCrdCnt[i]);
			break;
		case 2 :
			Serial_Printf((CPU_CHAR *)"[%s%2d]", (CPU_CHAR *)"", Computer.nCrdCnt[i]);
		    break;
		case 3 :
			Serial_Printf((CPU_CHAR *)"[%s%2d]", "", Computer.nCrdCnt[i]);
		    break;
		default:
		    break;
		} // switch END
		i++;

	} // while computer_card[i] END

	Serial_Printf((CPU_CHAR *)"\n\r");
	i = 0;

	Serial_Printf((CPU_CHAR *)"Player Card :");
	while (Player.nCrdCnt[i] != 0){

		// ī    (ڿ )
		switch(Computer.nCrdKnd[i]){
		case 0 :
			Serial_Printf((CPU_CHAR *)"[%s%2d]", (CPU_CHAR *)"", Player.nCrdCnt[i]);
			break;
		case 1 :
			Serial_Printf((CPU_CHAR *)"[%s%2d]", (CPU_CHAR *)"", Player.nCrdCnt[i]);
			break;
		case 2 :
			Serial_Printf((CPU_CHAR *)"[%s%2d]", (CPU_CHAR *)"", Player.nCrdCnt[i]);
		    break;
		case 3 :
			Serial_Printf((CPU_CHAR *)"[%s%2d]", (CPU_CHAR *)"", Player.nCrdCnt[i]);
		    break;
		default:
		    break;
		} // switch END
		i++;

	} // while player_card[i] END
	Serial_Printf((CPU_CHAR *)"\n\r");

	return 0;
} // PrtCard END


/* ++LSM
*********************************************************************************************************
*                                        Judge(void)
*
* Description :    ϴ Լ
*
* Argument(s) : none.
*
* Note(s)     : (1)
*********************************************************************************************************
*/
int Judge(void)
{
	int i;
	int nComTmp = 0;
	int nPlyTmp = 0;

	for ( i = 0 ; i < USEDCARD ; i++){
		nComTmp += Computer.nCrdCnt[i];
		nPlyTmp += Player.nCrdCnt[i];
	} // for i END

	// ī  21 ʰ ...
	if ( nComTmp % 22 >= nPlyTmp % 22){
		Serial_Printf((CPU_CHAR *)"Dealer Win~~~!!!\n\r");
		Computer.nPoint++;
		Player.nPoint--;
	} // if nComTmp END
	else{
		Serial_Printf((CPU_CHAR *)"Player Win~~~!!!\n\r");
		Computer.nPoint--;
		Player.nPoint++;
	} // else END

	return 0;
} // Judge END
