// 
//==============================================
// ETK(Embedded Training Kit) - AT91SAM7SE512
//==============================================
//
// File name : bsp.c
// Project name : ETK-uCOSii
// Description : Board Suport Package for AT91SAM7SE512-EK, not ETK board.
// By : Roh, YoungSup(ysroh@suv.ac.kr)
//      Dept. of Embedded Systems. 
//      SUV ( Seoul University of Venture and Information )
// Date : 2007.12.14
// Modifications :
//     1. OS_PROBE ִ app.c  Ͽ.
//     2. Probe شǴ  ڵ .
//     3. Joystick شǴ  ڵ .
//     4. PushButton  ڵ 
//     5. PLL_Init() local global .
//         cstartup.S 
//     6.  #define xxx etk.h ̵, GPIOA_LED_POWER ETK_LED0 , GPIOx_ ETK_ .
//     7. etk.c ETK_Init() ȣ
//
//===============================================
//


/*
*********************************************************************************************************
*                                     MICIRUM BOARD SUPPORT PACKAGE
*
*                          (c) Copyright 2003-2006; Micrium, Inc.; Weston, FL
*
*                   All rights reserved.  Protected by international copyright laws.
*                   Knowledge of the source code may not be used to write a similar
*                   product.  This file may only be used in accordance with a license
*                   and should not be redistributed in any way.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                         BOARD SUPPORT PACKAGE
*
*                                           Atmel AT91SAM7SE
*                                                on the
*                                     AT91SAM7SE-EK Evaluation Board
*
* Filename      : bsp.c
* Version       : V1.00
* Programmer(s) : BAN
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#define  BSP_GLOBALS
#include <includes.h>


/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*/

#define  BSP_RAM_REMAP_TEST_BYTE             (*(CPU_INT08U  *)0x00000030L)

/*
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static  void  Tmr_TickInit          (void);

static  void  BSP_DummyISR_Handler  (void);

static  void  BSP_IntCtrlInit       (void);


/*
*********************************************************************************************************
*                                     LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*********************************************************************************************************
**                                          GLOBAL FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             BSP_Init()
*
* Description : Initialize the Board Support Package (BSP).
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s)     : (1) This function SHOULD be called before any other BSP function is called.
*********************************************************************************************************
*/

void  BSP_Init (void)
{
    AT91C_BASE_WDTC->WDTC_WDMR |= AT91C_WDTC_WDDIS;              /* Disable the Watchdog Timer                               */

                                                                /* Enable the hardware reset button                         */
    AT91C_BASE_RSTC->RSTC_RMR  |= (CPU_INT32U)(0xA5 << 24) | AT91C_RSTC_URSTEN;

    PLL_Init();                                                 /* Initialize the PLL and select it for MCLK                */

    LED_Init();                                                 /* Initialize the I/Os for the LEDs                         */

// -- roh
//	PB_Init();                                                  /* Initialize the I/Os for the push buttons                 */
//  Joystick_Init();                                            /* Initialize the I/Os for the joystick                     */

    BSP_IntCtrlInit();                                          /* Initialize the Interrupt Controller                      */
    Tmr_TickInit();                                             /* Initialize uC/OS-II's Tick Rate                          */

// ++ roh
    ETK_Init();													// Initialize the ETK-BSP
}


/*
*********************************************************************************************************
*                                            BSP_CPU_ClkFreq()
*
* Description : Get the master clock frequency (MCK) (which is also the processor clock frequency (PCK)).
*
* Argument(s) : none.
*
* Return(s)   : The CPU clock frequency, in Hz.
*********************************************************************************************************
*/

CPU_INT32U  BSP_CPU_ClkFreq (void)
{
    CPU_INT32U  mckr_css;
    CPU_INT32U  pll_mult;
    CPU_INT32U  pll_div;
    CPU_INT32U  mclk_div;
    CPU_INT32U  cpu_freq;

                                                                /* ------------------ MASTER CLOCK INPUT ------------------ */
    mckr_css = (AT91C_BASE_PMC->PMC_MCKR) & 0x00000003;

    switch (mckr_css) {

        case 0x00:                                              /* Slow clock                                               */
             cpu_freq = SLOW_XTAL_FREQ;
             break;

        case 0x01:                                              /* Main clock                                               */
             cpu_freq = MAIN_XTAL_FREQ;
             break;

        case 0x02:                                              /* Reserved                                                 */
             cpu_freq = 0;
             break;

        case 0x03:                                              /* PLL clock                                                */
             pll_mult = (AT91C_BASE_CKGR->CKGR_PLLR & 0x07FF0000) >> 16;
             pll_div  = (AT91C_BASE_CKGR->CKGR_PLLR & 0x000000FF) >>  0;

             if (pll_div == 0) {
                 cpu_freq = 0;                                  /* If PLL divider is zero, then PLL output is zero.         */
             } else if (pll_mult == 0) {
                 cpu_freq = MAIN_XTAL_FREQ;                     /* If PLL multiplier is zero, then PLL is disabled.         */
             } else {
                 cpu_freq = (MAIN_XTAL_FREQ / pll_div) * (pll_mult + 1);
             }
             break;
    }
                                                                /* ---------------- MASTER CLOCK PRESCALER ---------------- */
                                                                /* Read the Master Clock divider                            */
    mclk_div = (AT91C_BASE_PMC->PMC_MCKR >> 2) & 0x07;

    mclk_div = 1 << mclk_div;                                   /* Convert 0-7 into 1, 2, 4, 8, 16, 32, or 64               */

    if (mclk_div >= 128) {                                      /* Divider pattern for 128 is reserved                      */
        return (cpu_freq);
    }

    cpu_freq = cpu_freq / mclk_div;

    return (cpu_freq);
}


/*
*********************************************************************************************************
*                                          OS_CPU_ExceptHndlr()
*
* Description : Handle any exceptions.
*
* Argument(s) : except_id     ARM exception type:
*
*                                  OS_CPU_ARM_EXCEPT_RESET             0x00
*                                  OS_CPU_ARM_EXCEPT_UNDEF_INSTR       0x01
*                                  OS_CPU_ARM_EXCEPT_SWI               0x02
*                                  OS_CPU_ARM_EXCEPT_PREFETCH_ABORT    0x03
*                                  OS_CPU_ARM_EXCEPT_DATA_ABORT        0x04
*                                  OS_CPU_ARM_EXCEPT_ADDR_ABORT        0x05
*                                  OS_CPU_ARM_EXCEPT_IRQ               0x06
*                                  OS_CPU_ARM_EXCEPT_FIQ               0x07
*
* Return(s)   : none.
*
* Caller(s)   : OS_CPU_ARM_EXCEPT_HANDLER(), which is declared in os_cpu_a.s.
*********************************************************************************************************
*/

void  OS_CPU_ExceptHndlr (CPU_INT32U  except_id)
{
    CPU_FNCT_VOID   pfnct;
    CPU_INT32U     *sp;


    if (except_id == OS_CPU_ARM_EXCEPT_IRQ) {
        pfnct = (CPU_FNCT_VOID)AT91C_BASE_AIC->AIC_IVR;         /* Read the interrupt vector from the VIC                   */

        while (pfnct != (CPU_FNCT_VOID)0) {                     /* Make sure we don't have a NULL pointer                   */

          (*pfnct)();                                           /* Execute the ISR for the interrupting device              */
            AT91C_BASE_AIC->AIC_EOICR = 0;                      /* End of handler                                           */
            pfnct = (CPU_FNCT_VOID)(AT91C_BASE_AIC->AIC_IVR);   /* Read IRQ hanlder from the AIC                            */
        }

        AT91C_BASE_AIC->AIC_EOICR = 0;                          /* End of handler                                           */

    } else if (except_id == OS_CPU_ARM_EXCEPT_FIQ) {
        pfnct = (CPU_FNCT_VOID)AT91C_BASE_AIC->AIC_FVR;         /* Read the interrupt vector from the VIC                   */

        while (pfnct != (CPU_FNCT_VOID)0) {                     /* Make sure we don't have a NULL pointer                   */

          (*pfnct)();                                           /* Execute the ISR for the interrupting device              */
            AT91C_BASE_AIC->AIC_EOICR = 0;                      /* End of handler                                           */
            pfnct = (CPU_FNCT_VOID)(AT91C_BASE_AIC->AIC_FVR);   /* Read FIQ handler from the AIC                            */
        }

        AT91C_BASE_AIC->AIC_EOICR = 0;                          /* End of handler                                           */

    } else {
        sp = (CPU_INT32U *)OSTCBCur->OSTCBStkPtr;
        APP_TRACE_INFO(("\nCPU_ARM_EXCEPTION #%d trapped.\n", except_id));
        APP_TRACE_INFO(("R0  : 0x%08x\n", *(sp + 0x01)));
        APP_TRACE_INFO(("R1  : 0x%08x\n", *(sp + 0x02)));
        APP_TRACE_INFO(("R2  : 0x%08x\n", *(sp + 0x03)));
        APP_TRACE_INFO(("R3  : 0x%08x\n", *(sp + 0x04)));
        APP_TRACE_INFO(("R4  : 0x%08x\n", *(sp + 0x05)));
        APP_TRACE_INFO(("R5  : 0x%08x\n", *(sp + 0x06)));
        APP_TRACE_INFO(("R6  : 0x%08x\n", *(sp + 0x07)));
        APP_TRACE_INFO(("R7  : 0x%08x\n", *(sp + 0x08)));
        APP_TRACE_INFO(("R8  : 0x%08x\n", *(sp + 0x09)));
        APP_TRACE_INFO(("R9  : 0x%08x\n", *(sp + 0x0A)));
        APP_TRACE_INFO(("R10 : 0x%08x\n", *(sp + 0x0B)));
        APP_TRACE_INFO(("R11 : 0x%08x\n", *(sp + 0x0C)));
        APP_TRACE_INFO(("R12 : 0x%08x\n", *(sp + 0x0D)));
        APP_TRACE_INFO(("SP  : 0x%08x\n",   sp));
        APP_TRACE_INFO(("LR  : 0x%08x\n", *(sp + 0x0E)));
        APP_TRACE_INFO(("PC  : 0x%08x\n", *(sp + 0x0F)));
        APP_TRACE_INFO(("CPSR: 0x%08x\n", *(sp + 0x00)));

                                                                /* Infinite loop on other exceptions.                       */
                                                                /* Should be replaced by other behavior (reboot, etc.)      */
        while (DEF_TRUE) {
            ;
        }
    }
}


/*
*********************************************************************************************************
*                                           BSP_IntDisAll()
*
* Description : Disable ALL interrupts.
*
* Argument(s) : none.
*
* Return(s)   : none.
*********************************************************************************************************
*/

void  BSP_IntDisAll (void)
{
    AT91C_BASE_AIC->AIC_IDCR = 0xFFFFFFFF;                              /* Disable all interrupts                                   */
}


/*
******************************************************************************************************************************
******************************************************************************************************************************
**                                       LED, PB & JOYSTICK FUNCTIONS
******************************************************************************************************************************
******************************************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             LED_Init()
*
* Description : Setup the I/O for the LEDs.
*
* Argument(s) : none.
*
* Return(s)   : none.
*********************************************************************************************************
*/

void  LED_Init (void)
{
    AT91C_BASE_PMC->PMC_PCER = 1 << AT91C_ID_PIOA;                          /* Enable peripheral clock                      */
	
//    AT91C_BASE_PIOA->PIO_PER = GPIOA_LED1 | GPIOA_LED2 | GPIOA_LED0;   /* Enable PIO control of pins                   */
//    AT91C_BASE_PIOA->PIO_OER = GPIOA_LED1 | GPIOA_LED2 | GPIOA_LED0;   /* Enable output on pins                        */
//    AT91C_BASE_PIOA->PIO_IDR = GPIOA_LED1 | GPIOA_LED2 | GPIOA_LED0;   /* Disable interrupt on pins                    */

    AT91C_BASE_PIOA->PIO_PER = ETK_LED0 | ETK_LED1 | ETK_LED2;   /* Enable PIO control of pins                   */
    AT91C_BASE_PIOA->PIO_OER = ETK_LED0 | ETK_LED1 | ETK_LED2;   /* Enable output on pins                        */
    AT91C_BASE_PIOA->PIO_IDR = ETK_LED0 | ETK_LED1 | ETK_LED2;   /* Disable interrupt on pins                    */

    LED_Off(0);                                                             /* Turn OFF all the LEDs                        */

}

/*
*********************************************************************************************************
*                                             LED_On()
*
* Description : Turn ON any or all the LEDs on the board.
*
* Argument(s) : led     The ID of the LED to control:
*
*                       0    turn ON all LEDs on the board
*                       1    turn ON LED0 on the board
*                       2    turn ON LED1 on the board
*                       3    turn ON LED2 on the board
*
* Return(s)   : none.
*********************************************************************************************************
*/

void  LED_On (CPU_INT08U led)
{
    switch (led) {
        case 0:
             AT91C_BASE_PIOA->PIO_CODR = ETK_LED0 | ETK_LED1 | ETK_LED2;
             break;

        case 1:
             AT91C_BASE_PIOA->PIO_CODR = ETK_LED0;
             break;

        case 2:
             AT91C_BASE_PIOA->PIO_CODR = ETK_LED1;
             break;

        case 3:
             AT91C_BASE_PIOA->PIO_CODR = ETK_LED2;
             break;
    }
}

/*
*********************************************************************************************************
*                                             LED_Off()
*
* Description : Turn OFF any or all the LEDs on the board.
*
* Argument(s) : led     The ID of the LED to control:
*
*                       0    turn OFF all LEDs on the board
*                       1    turn OFF LED0 on the board
*                       2    turn OFF LED1 on the board
*                       3    turn OFF LED2 on the board
*
* Return(s)   : none.
*********************************************************************************************************
*/

void  LED_Off (CPU_INT08U led)
{
    switch (led) {
        case 0:
             AT91C_BASE_PIOA->PIO_SODR = ETK_LED0 | ETK_LED1 | ETK_LED2;
             break;

        case 1:
             AT91C_BASE_PIOA->PIO_SODR = ETK_LED0;
             break;

        case 2:
             AT91C_BASE_PIOA->PIO_SODR = ETK_LED1;
             break;

        case 3:
             AT91C_BASE_PIOA->PIO_SODR = ETK_LED2;
             break;
    }
}

/*
*********************************************************************************************************
*                                             LED_Toggle()
*
* Description : TOGGLE any or all the LEDs on the board.
*
* Argument(s) : led     The ID of the LED to control:
*
*                       0    turn TOGGLE all LEDs on the board
*                       1    turn TOGGLE LED1 on the board
*                       2    turn TOGGLE LED2 on the board
*                       3    turn TOGGLE LED3 on the board (the power LED)
*
* Return(s)   : none.
*********************************************************************************************************
*/

void  LED_Toggle (CPU_INT08U led)
{
    switch (led) {
        case 0:
             if (AT91C_BASE_PIOA->PIO_ODSR & ETK_LED1) {
                 AT91C_BASE_PIOA->PIO_CODR = ETK_LED1;
             } else {
                 AT91C_BASE_PIOA->PIO_SODR = ETK_LED1;
             }
             if (AT91C_BASE_PIOA->PIO_ODSR & ETK_LED2) {
                 AT91C_BASE_PIOA->PIO_CODR = ETK_LED2;
             } else {
                 AT91C_BASE_PIOA->PIO_SODR = ETK_LED2;
             }
             if (AT91C_BASE_PIOA->PIO_ODSR & ETK_LED0) {
                 AT91C_BASE_PIOA->PIO_CODR = ETK_LED0;
             } else {
                 AT91C_BASE_PIOA->PIO_SODR = ETK_LED0;
             }
             break;

        case 1:
             if (AT91C_BASE_PIOA->PIO_ODSR & ETK_LED1) {
                 AT91C_BASE_PIOA->PIO_CODR = ETK_LED1;
             } else {
                 AT91C_BASE_PIOA->PIO_SODR = ETK_LED1;
             }
             break;

        case 2:
             if (AT91C_BASE_PIOA->PIO_ODSR & ETK_LED2) {
                 AT91C_BASE_PIOA->PIO_CODR = ETK_LED2;
             } else {
                 AT91C_BASE_PIOA->PIO_SODR = ETK_LED2;
             }
             break;
        case 3:
             if (AT91C_BASE_PIOA->PIO_ODSR & ETK_LED0) {
                 AT91C_BASE_PIOA->PIO_CODR = ETK_LED0;
             } else {
                 AT91C_BASE_PIOA->PIO_SODR = ETK_LED0;
             }
             break;
    }
}



/*
*********************************************************************************************************
*********************************************************************************************************
**                                          uC/OS-View FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                              OSView_TmrInit()
*
* Description : Select & initialize a timer for use with uC/OS-View.
*
* Argument(s) : none.
*
* Return(s)   : none.
*********************************************************************************************************
*/

#if (OS_VIEW_MODULE > 0)
void  OSView_TmrInit (void)
{
#if (OS_VIEW_TIMER_SEL == 0)
    AT91C_BASE_PMC->PMC_PCER        =  1 << AT91C_ID_TC0;       /* Enable the peripheral clk                                */
    AT91C_BASE_TCB->TCB_TC0.TC_CCR  =  DEF_BIT_01;              /* TC0 timer disabled                                       */
    AT91C_BASE_TCB->TCB_TC0.TC_CMR &= ~(7 <<  0);               /* TIMER_CLOCK1 is input clk                                */
    AT91C_BASE_TCB->TCB_TC0.TC_CCR  =  DEF_BIT_00;              /* TC0 timer enabled                                        */
    AT91C_BASE_TCB->TCB_TC0.TC_CCR  =  DEF_BIT_02;              /* SWTRG to reset and start                                 */
#endif

#if (OS_VIEW_TIMER_SEL == 1)
    AT91C_BASE_PMC->PMC_PCER        =  1 << AT91C_ID_TC1;       /* Enable the peripheral clk                                */
    AT91C_BASE_TCB->TCB_TC1.TC_CCR  =  DEF_BIT_01;              /* TC1 timer disabled                                       */
    AT91C_BASE_TCB->TCB_TC1.TC_CMR &= ~(7 <<  0);               /* TIMER_CLOCK1 is input clk                                */
    AT91C_BASE_TCB->TCB_TC1.TC_CCR  =  DEF_BIT_00;              /* TC1 timer enabled                                        */
    AT91C_BASE_TCB->TCB_TC1.TC_CCR  =  DEF_BIT_02;              /* SWTRG to reset and start                                 */
#endif

#if (OS_VIEW_TIMER_SEL == 2)
    AT91C_BASE_PMC->PMC_PCER        =  1 << AT91C_ID_TC2;       /* Enable the peripheral clk                                */
    AT91C_BASE_TCB->TCB_TC2.TC_CCR  =  DEF_BIT_01;              /* TC2 timer disabled                                       */
    AT91C_BASE_TCB->TCB_TC2.TC_CMR &= ~(7 <<  0);               /* TIMER_CLOCK1 is input clk                                */
    AT91C_BASE_TCB->TCB_TC2.TC_CCR  =  DEF_BIT_00;              /* TC2 timer enabled                                        */
    AT91C_BASE_TCB->TCB_TC2.TC_CCR  =  DEF_BIT_02;              /* SWTRG to reset and start                                 */
#endif

}
#endif


/*
*********************************************************************************************************
*                                         OSView_TmrRd()
*
* Description : Read the current counts of a 16-bit free running timer.
*
* Argument(s) : none.
*
* Return(s)   : The 16 bit counts (in a 32 bit variable) of the timer.
*********************************************************************************************************
*/

#if (OS_VIEW_MODULE > 0)
CPU_INT32U  OSView_TmrRd (void)
{
    CPU_INT32U  cnts;


#if (OS_VIEW_TIMER_SEL == 0)
    cnts = (CPU_INT32U)(AT91C_BASE_TCB->TCB_TC0.TC_CV & 0x0000FFFF);    /* Read timer 0                                     */
#endif

#if (OS_VIEW_TIMER_SEL == 1)
    cnts = (CPU_INT32U)(AT91C_BASE_TCB->TCB_TC1.TC_CV & 0x0000FFFF);    /* Read timer 1                                     */
#endif

#if (OS_VIEW_TIMER_SEL == 2)
    cnts = (CPU_INT32U)(AT91C_BASE_TCB->TCB_TC2.TC_CV & 0x0000FFFF);    /* Read timer 2                                     */
#endif

    return (cnts);
}
#endif



/*
*********************************************************************************************************
*********************************************************************************************************
**                                     uC/OS-II TIMER FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                            Tmr_TickInit()
*
* Description : Initialize uC/OS-II's tick source; for the AT91SAM, the Periodic Interval Timer (PIT).
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s)     : (1) PIT Interrupt frequency:
*
*                             MCLK        1
*                   Freq =    ---- * -----------
*                              16     (PIV + 1)
*
*
*                             MCLK      1
*                   PIV  =  ( ---- * ------ ) - 1
*                              16     Freq
*
*                   Where:
*
*                       MCLK = Master Clock
*                       Freq = Desired frequency (i.e. OS_TICKS_PER_SEC)
*********************************************************************************************************
*/

static  void  Tmr_TickInit (void)
{
    CPU_INT32U  counts;
    CPU_INT32U  cpu_freq;

                                                                /* Determine the number of counts per tick                  */
    cpu_freq                              = BSP_CPU_ClkFreq();

    counts                                = ((cpu_freq) / 16 / OS_TICKS_PER_SEC) - 1;

                                                                /* Set the vector address for PIT                           */
    AT91C_BASE_AIC->AIC_SVR[AT91C_ID_SYS] = (CPU_INT32U)Tmr_TickISR_Handler;
    AT91C_BASE_AIC->AIC_SMR[AT91C_ID_SYS] = AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL
                                          | AT91C_AIC_PRIOR_LOWEST;

    AT91C_BASE_AIC->AIC_ICCR              = 1 << AT91C_ID_SYS;
    AT91C_BASE_AIC->AIC_IECR              = 1 << AT91C_ID_SYS;

                                                                /* Enable the PIT with the correct compare value            */
    AT91C_BASE_PITC->PITC_PIMR            = AT91C_PITC_PITEN | AT91C_PITC_PITIEN | counts;
}


/*
*********************************************************************************************************
*                                       Tmr_TickISR_Handler()
*
* Description : Handle the PIT interrupt that is used to generate TICKs for uC/OS-II.
*
* Argument(s) : none.
*
* Return(s)   : none.
*********************************************************************************************************
*/

// cc roh
//static  void  Tmr_TickISR_Handler (void)
void  Tmr_TickISR_Handler (void)
{
    CPU_INT32U  status;


    AT91C_BASE_AIC->AIC_IVR  = 0;                               /* Write the IVR, as required in Protection Mode            */

    status                   = AT91C_BASE_PITC->PITC_PIVR;
    (void)status;                                               /* Prevent compiler warning about status set & not used     */

    OSTimeTick();                                               /* Tell uC/OS-II about clock tick                           */

    AT91C_BASE_AIC->AIC_ICCR = 1 << AT91C_ID_SYS;
}


/*
*********************************************************************************************************
*********************************************************************************************************
**                                       Serial Port Communications
*********************************************************************************************************
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                                   Ser_Init()
*
* Description : Initialize a serial port for communication.
*
* Argument(s) : none.
*
* Return(s)   : none.
*********************************************************************************************************
*/

void  Ser_Init (void)
{
    CPU_INT32U  pclk_freq;


    pclk_freq                  = BSP_CPU_ClkFreq();             /* Get the UART input clock frequency                       */

    AT91C_BASE_PIOA->PIO_PDR   = ETK_DRXD | ETK_DTXD;       /* Set GPIOA pins 9 and 10 as DBGU UART pins                */
    AT91C_BASE_PIOA->PIO_ASR   = ETK_DRXD | ETK_DTXD;       /* Select GPIOA attached peripheral (DBGU)                  */

    AT91C_BASE_DBGU->DBGU_IDR  = AT91C_US_RXRDY                 /* Disable Rx interrupts                                    */
                               | AT91C_US_TXRDY;                /* Disable Tx interrupt                                     */

    AT91C_BASE_DBGU->DBGU_CR   = AT91C_US_RXEN                  /* Enable the receiver                                      */
                               | AT91C_US_TXEN;                 /* Enable the transmitter                                   */

    AT91C_BASE_DBGU->DBGU_MR   = AT91C_US_USMODE_NORMAL         /* Normal mode selected                                     */
                               | AT91C_US_PAR_NONE;             /* No parity bit selected                                   */

                                                                /* Set the DBGU baud rate to 115,200                        */
    AT91C_BASE_DBGU->DBGU_BRGR = (CPU_INT16U)((pclk_freq) / 115200 / 16);

    AT91C_BASE_PMC->PMC_PCER   = (1 << AT91C_ID_SYS);           /* Enable the DBGU peripheral clock                         */
}


/*
*********************************************************************************************************
*                                                Ser_WrByte()
*
* Description : Writes a single byte to a serial port.
*
* Argument(s) : tx_byte     The character to output.
*
* Return(s)   : none.
*
* Note(s)     : (1) This functino blocks until room is available in the UART for the byte to be sent.
*********************************************************************************************************
*/

void  Ser_WrByte (CPU_INT08U tx_byte)
{
    while ((AT91C_BASE_DBGU->DBGU_CSR & AT91C_US_TXRDY) == 0){  /*  Wait for room in the transmit register.                 */
        ;
    }

    AT91C_BASE_DBGU->DBGU_THR     = tx_byte;
}


/*
*********************************************************************************************************
*                                                Ser_WrStr()
*
* Description : Write a character string to a serial port.
*
* Argument(s) : tx_str      A character string.
*
* Return(s)   : none.
*********************************************************************************************************
*/

void  Ser_WrStr (CPU_CHAR *tx_str)
{
    while ((*tx_str) != 0) {
        Ser_WrByte(*tx_str++);
    }
}


/*
*********************************************************************************************************
*                                                Ser_RdByte()
*
* Description : Read a byte from a serial port and echo byte to port.
*
* Argument(s) : none.
*
* Return(s)   : A byte containing the value of the received charcater.
*
* Note(s)     : (1) This function blocks until a character appears at the port.
*********************************************************************************************************
*/

CPU_INT08U Ser_RdByte (void)
{
    CPU_INT08U      rx_byte;

    while ((AT91C_BASE_DBGU->DBGU_CSR & AT91C_US_RXRDY) == 0) {     /*  Wait for a byte to show up.                         */
        OSTimeDly(2);
    }
    rx_byte = (CPU_INT08U)(AT91C_BASE_DBGU->DBGU_RHR & 0x00FF);     /* Read the character.                                  */
    return (rx_byte);
}


/*
*********************************************************************************************************
*                                                Ser_RdStr()
*
* Description : Read a string from a serial port.
*
* Argument(s) : rx_str      A pointer to a buffer at which the string can be stored.
*               len         The size of the string that will be read.
*
* Return(s)   : none.
*********************************************************************************************************
*/

void  Ser_RdStr (CPU_CHAR    *rx_str,
                 CPU_INT32U   len)
{
    CPU_CHAR  input;
    CPU_CHAR  input_ix;


    input_ix  = 0;
    rx_str[0] = 0;

    while (1)
    {
        input = Ser_RdByte();



        if ((input == '\r') ||
            (input == '\n')) {
            Ser_Printf((CPU_CHAR *)"\n");
            rx_str[input_ix] = 0;
            break;
        }

        if (input == '\b') {
            if (input_ix > 0) {
                Ser_Printf((CPU_CHAR *)"\b \b");
                input_ix--;
                rx_str[input_ix] = 0;
            }
        }

        if (Str_IsPrint(input)) {
            Ser_Printf((CPU_CHAR *)"%c", input);
            rx_str[input_ix] = input;
            input_ix++;
            if (input_ix >= len) {
               input_ix = len;
            }
        }

    }
}


/*
*********************************************************************************************************
*                                                Ser_Printf()
*
* Description : Formatted outout to the serial port.
*
* Argument(s) : format      Format string follwing the C format convention.
*
* Return(s)   : none.
*********************************************************************************************************
*/


void  Ser_Printf (CPU_CHAR *format, ...)
{
    static  CPU_CHAR  buffer[80 + 1];
            va_list   vArgs;


    va_start(vArgs, format);


	vsprintf((char *)buffer, (char const *)format, vArgs);

    va_end(vArgs);

    Ser_WrStr((CPU_CHAR*) buffer);
}

/*
*********************************************************************************************************
*********************************************************************************************************
**                                          LOCAL FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                        BSP_DummyISR_Handler()
*
* Description : Handle an invalid IRQ.
*
* Argument(s) : none.
*
* Return(s)   : none.
*********************************************************************************************************
*/

static  void  BSP_DummyISR_Handler (void)
{
    CPU_INT32U  irq_id;


    irq_id                   = AT91C_BASE_AIC->AIC_ISR & 0x1F;  /* Retrieve the ID of the interrupting source           */
    AT91C_BASE_AIC->AIC_ICCR =  (1 << irq_id);                  /* Clear the current interrupt                          */
    AT91C_BASE_AIC->AIC_IVR  = 0;                               /* Debug variant of vector read (protect mode is used)  */
}


/*
*********************************************************************************************************
*                                          PLL_Init()
*
* Description : Iinitializes the PLL & sets the proper number of flash wait states.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s)
*   (1) Do not use external functions such as printf(), since this function is called in cstartup.S
*
*********************************************************************************************************
*/

// cc roh
//static  void  PLL_Init (void)
void  PLL_Init (void)
{
                                                                /* ----------------- SET FLASH WAIT STATES ---------------- */
    AT91C_BASE_MC->MC0_FMR   |= AT91C_MC_FWS_2FWS;              /* Since we are running > 30mhz out of flash                */
    AT91C_BASE_MC->MC1_FMR   |= AT91C_MC_FWS_2FWS;              /* Since we are running > 30mhz out of flash                */

                                                                /* -------------------- SETUP CPU CLOCK ------------------- */
                                                                /* Enable the Main Oscillator                               */
    AT91C_BASE_PMC->PMC_MOR   = AT91C_CKGR_OSCOUNT | AT91C_CKGR_MOSCEN;

                                                                /* Wait for the Main Oscillator to start up                 */
    while ((AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MOSCS) == 0) {
        ;
    }

    AT91C_BASE_PMC->PMC_PLLR  = 0x107C3F0C;                     /* Enable and set PLL                                       */
                                                                /* [07:00]  Divider     = 12                                */
                                                                /* [13:08]  PLL Count   = 63 slow clocks                    */
                                                                /* [26:16]  Multiplier  = (124 + 1) = 125                   */
                                                                /* [29:28]  USB Divider = 2                                 */
                                                                /* ===> MCLK   = 18432000 * 125 / 12 = 192 MHz              */
                                                                /* ===> USBCLK = MCLK / 4            =  48 MHz              */

                                                                /* Wait for the PLL to start up                             */
    while ((AT91C_BASE_PMC->PMC_SR & AT91C_PMC_LOCK) == 0) {
        ;
    }

    while((AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY) == 0) {
        ;
    }

                                                                /* Main Clock will be equal to 1/4 of the PLL Clock         */
    AT91C_BASE_PMC->PMC_MCKR  =  AT91C_PMC_PRES_CLK_4;

    while((AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY) == 0) {
        ;
    }
                                                                /* Specify the PLL Clock as the source of the Main Clock    */
    AT91C_BASE_PMC->PMC_MCKR |= AT91C_PMC_CSS_PLL_CLK;

    while((AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY) == 0) {
        ;
    }
}


/*
*********************************************************************************************************
*                                       BSP_IntCtrlInit()
*
* Description : (1) Initialize interrupts:
*                    (a) Initialize exception vector table, performing a remap (if necessary).
*                    (b) Initialize vectors in Advance Interrupt Controller (AIC) to dummy ISR.
*
* Argument(s) : none.
*
* Return(s)   : none.
*********************************************************************************************************
*/

static  void  BSP_IntCtrlInit (void)
{
    CPU_INT16U  i;


    BSP_RAM_REMAP_TEST_BYTE = 0xAA;                             /* Write a byte to RAM                                      */

#ifndef  RAM_REMAPPED
    if (BSP_RAM_REMAP_TEST_BYTE == 0xAA) {                      /* Check if the write to RAM worked v                       */
        AT91C_BASE_MC->MC_RCR =  1;                             /* If so, toggle REMAP register                             */
    }
#else                                                           /*--------------BEGIN CODE RUNNING OUT OF RAM---------------*/
    OS_CPU_InitExceptVect();
#endif                                                          /*---------------END CODE RUNNING OUT OF RAM----------------*/

    AT91C_BASE_AIC->AIC_EOICR =  0x00000000;                    /* End-of-interrupt command                                 */

    for (i = 0; i < 32; i++) {                                  /* Disable all ISRs                                         */
        AT91C_BASE_AIC->AIC_SVR[i] = (CPU_INT32U)BSP_DummyISR_Handler;
        AT91C_BASE_AIC->AIC_SMR[i] = 0;
    }
}
