// 
//==============================================
// ETK(Embedded Training Kit) - AT91SAM7SE512
//==============================================
//
// File name : cstartup.S <-- cstartup.s
// Project name : ETK-uCOSii
// Description : Generic ARM Port
// By : Roh, YoungSup(ysroh@suv.ac.kr)
//      Dept. of Embedded Systems. 
//      SUV ( Seoul University of Venture and Information )
// Date : 2007.12.14
// Modifications :
//    1. Toolchain : IAR EWARM ==> GNU
//    2. Setup variables : data, bss
//    3. CONTINUE TO main() in app.c
//    4. PLL_Init() Լ ȣ - PLL  Ͽ[8 ׸  2008.02.16 ̻]
//    5. Ext_SRAM_Init() Լ ȣ - External SRAM ʱȭ
//    6. Disable Watchdog Timer
//    7. #define  etk.h ̵
//		8. PLL_Init() ȣ   ó ǵ [2008.02.16 ̻(golsm21@naver.com)]
//
//===============================================
//

#include <ioat91sam7se512.h>
#include <etk.h>
//
//********************************************************************************************************
//                                    EXCEPTION VECTORS & STARTUP CODE
//
// File      : cstartup.s
// For       : ARM7 or ARM9
// Toolchain : IAR EWARM V5.10 and higher
//********************************************************************************************************
//
	
//********************************************************************************************************
//                                           MACROS AND DEFINIITIONS
//********************************************************************************************************


.section  .startup

.code 32

//********************************************************************************************************
//                                            ARM EXCEPTION VECTORS
//********************************************************************************************************

    .global  __vector
    .global  __gnu_program_start

    .extern  OS_CPU_ARM_ExceptUndefInstrHndlr
    .extern  OS_CPU_ARM_ExceptSwiHndlr
    .extern  OS_CPU_ARM_ExceptPrefetchAbortHndlr
    .extern  OS_CPU_ARM_ExceptDataAbortHndlr
    .extern  OS_CPU_ARM_ExceptIrqHndlr
    .extern  OS_CPU_ARM_ExceptFiqHndlr


__vector:
    LDR	    PC, =__gnu_program_start                    // Absolute jump can reach 4 GByte
    LDR     PC, =OS_CPU_ARM_ExceptUndefInstrHndlr       // Branch to undef_handler
    LDR     PC, =OS_CPU_ARM_ExceptSwiHndlr              // Branch to swi_handler
    LDR     PC, =OS_CPU_ARM_ExceptPrefetchAbortHndlr    // Branch to prefetch_handler
    LDR     PC, =OS_CPU_ARM_ExceptDataAbortHndlr        // Branch to data_handler
    LDR     PC, =OS_CPU_ARM_Reserved                    // ++ roh Reserved
    LDR	    PC, =OS_CPU_ARM_ExceptIrqHndlr              // Branch to irq_handler
    LDR	    PC, =OS_CPU_ARM_ExceptFiqHndlr              // Branch to fiq_handler


//********************************************************************************************************
//                                   LOW-LEVEL INITIALIZATION
//********************************************************************************************************

    .extern  main

__gnu_program_start:


// ++ roh
//********************************************************************************************************
//                                    DISABLE Watchdog TImer
//********************************************************************************************************
	LDR	r0, =AT91C_WDTC_WDDIS
	LDR	r1, =AT91C_WDTC_WDMR
	STR	r0, [r1]


//********************************************************************************************************
//                                    STACK POINTER INITIALIZATION
//********************************************************************************************************

    MRS     r0,cpsr                             // Original PSR value
    BIC     r0,r0,#MODE_BITS                    // Clear the mode bits
    ORR     r0,r0,#SVC_MODE                     // Set SVC mode bits
    MSR     cpsr_c,r0                           // Change the mode
    LDR     sp,=SVC_STACK                  // End of SVC_STACK

    BIC     r0,r0,#MODE_BITS                    // Clear the mode bits
    ORR     r0,r0,#UND_MODE                     // Set UND mode bits
    MSR     cpsr_c,r0                           // Change the mode
    LDR     sp,=UND_STACK                  // End of UND_STACK

    BIC     r0,r0,#MODE_BITS                    // Clear the mode bits
    ORR     r0,r0,#ABT_MODE                     // Set ABT mode bits
    MSR     cpsr_c,r0                           // Change the mode
    LDR     sp,=ABT_STACK                  // End of ABT_STACK

    BIC     r0,r0,#MODE_BITS                    // Clear the mode bits
    ORR     r0,r0,#FIQ_MODE                     // Set FIQ mode bits
    MSR     cpsr_c,r0                           // Change the mode
    LDR     sp,=FIQ_STACK                  // End of FIQ_STACK

    BIC     r0,r0,#MODE_BITS                    // Clear the mode bits
    ORR     r0,r0,#IRQ_MODE                     // Set IRQ mode bits
    MSR     cpsr_c,r0                           // Change the mode
    LDR     sp,=IRQ_STACK                  // End of IRQ_STACK

    BIC     r0,r0,#MODE_BITS                    // Clear the mode bits
    ORR     r0,r0,#SYS_MODE                     // Set System mode bits
    MSR     cpsr_c,r0                           // Change the mode
    LDR     sp,=SYS_STACK                  // End of SYS_STACK


// ++ roh
//********************************************************************************************************
//                                   ADDITIONAL INITIALIZATION
//********************************************************************************************************

// ++ roh
//********************************************************************************************************
//  Set PLL to speed up
//
//     
//		PLL_Init() ȣ ϰ  ϴ  [ 8 ׸  2008.02.16]
//********************************************************************************************************
asm_pll_init:
		LDR			r0, =(AT91C_EFC0_FMR | AT91C_MC_FWS_2FWS)				// ÷ ޸   
		LDR			r1, =AT91C_EFC0_FMR
		STR			r0, [r1]
		 
		LDR			r0, =(AT91C_EFC1_FMR | AT91C_MC_FWS_2FWS)
		LDR			r1, =AT91C_EFC1_FMR
		STR			r0, [r1]
		
		LDR			r0, =(AT91C_CKGR_OSCOUNT | AT91C_CKGR_MOSCEN)		// Enable the main oscillator
		LDR			r1, =AT91C_PMC_MOR
		STR			r0, [r1]
		
		LDR			r1, =AT91C_PMC_SR																// Wate for the main oscillator to start up
wait_osc_stable:
		LDR			r0, [r1]
		ANDS		r0, r0, #AT91C_PMC_MOSCS
		BEQ			wait_osc_stable
		
		LDR			r0, =(0x107C3F0C)																// Enable and set PLL
		LDR			r1, =AT91C_PMC_PLLR															// [07:00]  Divider     = 12
		STR			r0, [r1]																				// [13:08]  PLL Count   = 63 slow clocks
																														// [26:16]  Multiplier  = (124 + 1) = 125
																														// [29:28]  USB Divider = 2
																														// ===> MCLK   = 18432000 * 125 / 12 = 192 MHz
																														// ===> USBCLK = MCLK / 4            =  48 MHz
		
		LDR			r1, =AT91C_PMC_SR																// Wait for the PLL to start up
wait_lock_stable:
		LDR			r0, [r1]
		ANDS		r0, r0, #AT91C_PMC_LOCK
		BEQ			wait_lock_stable

		LDR			r1, =AT91C_PMC_SR																
wait_mck_stable1:
		LDR			r0, [r1]
		ANDS		r0, r0, #AT91C_PMC_MCKRDY
		BEQ			wait_mck_stable1

		LDR			r0, =(AT91C_PMC_PRES_CLK_4)											
		LDR			r1, =AT91C_PMC_MCKR
		STR			r0, [r1]
		
		LDR			r1, =AT91C_PMC_SR																
wait_mck_stable2:
		LDR			r0, [r1]
		ANDS		r0, r0, #AT91C_PMC_MCKRDY
		BEQ			wait_mck_stable2
		
		LDR			r0, =(AT91C_PMC_PRES_CLK_4 | AT91C_PMC_CSS_PLL_CLK)		// Main Clock will be equal to 1/4 of the PLL Clock
		LDR			r1, =AT91C_PMC_MCKR
		STR			r0, [r1]
		
		LDR			r1, =AT91C_PMC_SR																// Specify the PLL Clock as the source of the Main Clock
wait_mck_stable3:
		LDR			r0, [r1]
		ANDS		r0, r0, #AT91C_PMC_MCKRDY
		BEQ			wait_mck_stable3
		
				
// ++ roh
//********************************************************************************************************
//  Call Ext_SRAM_Init() to use the external SRAM (512KB)
//
//     The Ext_SRAM_Init() is in etk.c
//********************************************************************************************************
asm_ext_sram_init:
   LDR      lr, =asm_ext_sram_init_end
   LDR      r0, =Ext_SRAM_Init
   BX       r0
asm_ext_sram_init_end:

// ++ roh
//********************************************************************************************************
//  Setup variables
//      data : initialized data
//          The initialized variables are relocated from ROM to RAM.
//      bss  : uninitialized data
//          The uninitialized variables are cleared.
//
//  *. ð  ɸ ۾ Ϸ System clock ؾ     ְ,
//  *. External Bus Interface Ͽ External SRAM Ϸ External SRAM interface ʱȭ ؾ Ѵ.
//
//********************************************************************************************************

// The inialized variables are relocated from ROM to RAM.
    LDR		R1, =_data_read_start
    LDR		R2, =_data_write_start
    LDR		R3, =_data_write_end
relocate_loop:
    CMP		R2,R3
    LDRLO	R0, [R1], #4
    STRLO	R0, [R2], #4
    BLO		relocate_loop

// Clear the BSS area
    MOV		R0, #0
    LDR		R1, =_bss_start
    LDR		R2, =_bss_end
bss_loop:
    CMP		R1, R2
    STRLO	R0, [R1], #4
    BLO		bss_loop


// cc roh
//********************************************************************************************************
//                           CONTINUE TO ?main FOR ADDITIONAL INITIALIZATION
//
//                           CONTINUE TO main() in app.c
//********************************************************************************************************

//  LDR     r0,=?main

    LDR     lr, =exit
    LDR     r0, =main
    BX      r0

exit:
    B       .

// ++ roh
OS_CPU_ARM_Reserved:
    B       .

//    END
