// 
//==============================================
// ETK(Embedded Training Kit) - AT91SAM7SE512
//==============================================
//
// File name : etk.c
// Project name : ETK-uCOSii
// Description : Board Suport Package for AT91SAM7SE512-EK, not EK board.
// By : Roh, YoungSup(ysroh@suv.ac.kr)
//      Dept. of Embedded Systems. 
//      SUV ( Seoul University of Venture and Information )
// Date : 2007.12.14
//
// Modifications : 
//     Rev 1.2 PWM    ߰(By JI(); ji77k7@hanmail.net)			2008.02.02
//     Rev 1.3 ETK_Init()  ׸"PWM_Init()" ߰(By LSM(̻); golsm21@naver.com)  2008.02.16 
//
//
//===============================================
//


/*
*********************************************************************************************************
*
*                                         BOARD SUPPORT PACKAGE
*
*                                           Atmel AT91SAM7SE
*                                                    on the
*                                        ETK-AT91SAM7SE512 Board
*
* Filename      : etk.c
* Version       : V1.00
* Programmer(s) : ysroh
*********************************************************************************************************
*/

#define  BSP_GLOBALS
#include <includes.h>

#define ESRAM_INIT_PAT 0x0000			// Initial value of SRAM area

static INT32U ext_sram_start = EXT_SRAM_ADDR_START;								//0x10000000
static INT32U ext_sram_end   = EXT_SRAM_ADDR_END;								//0x10080000, 32kBytes
static INT32U ext_sram_size  = EXT_SRAM_ADDR_END - EXT_SRAM_ADDR_START;			// 0x00080000, 512kBytes

/*
*********************************************************************************************************
*                                             Local functions
*********************************************************************************************************
*/

static INT8U Conv_To_7seg(INT8U c);
CPU_INT08U Serial_RdByte (void);

/*
*********************************************************************************************************
*                                             Local tables
*********************************************************************************************************
*/

// Matrix of seven segment
INT8U DBof7Seg[]={ _7SEG_0, _7SEG_1, _7SEG_2, _7SEG_3,
	               _7SEG_4, _7SEG_5, _7SEG_6, _7SEG_7,
	               _7SEG_8, _7SEG_9, _7SEG_A, _7SEG_B,
                      _7SEG_C, _7SEG_D, _7SEG_E, _7SEG_F};


/*
*********************************************************************************************************
*                                             ETK_Init()
*
* Description : Initialize the Embedded Training Kit(ETK).
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s)     : (1) This function SHOULD be called before use the ETK functions
*********************************************************************************************************
*/
void  ETK_Init (void)
{

    Ext_SRAM_Init();				// Initialize the External SRAM Interface
//  Do not use Ext_SRAM_Clean() here, if the "data" variables are moved to external SRAM in cstartup.S
    Ext_SRAM_Clean();				// Clear the External SRAM
    SevenSeg_Init();	
    Serial_Init();
    PWM_Init();
   		// Initialize the seven segment

}

CPU_INT32U  BSP_CPU_ClkFreq1 (void)
{
    CPU_INT32U  mckr_css;
    CPU_INT32U  pll_mult;
    CPU_INT32U  pll_div;
    CPU_INT32U  mclk_div;
    CPU_INT32U  cpu_freq;

                                                                /* ------------------ MASTER CLOCK INPUT ------------------ */
    mckr_css = (AT91C_BASE_PMC->PMC_MCKR) & 0x00000003;

    switch (mckr_css) {

        case 0x00:                                              /* Slow clock                                               */
             cpu_freq = SLOW_XTAL_FREQ;
             break;

        case 0x01:                                              /* Main clock                                               */
             cpu_freq = MAIN_XTAL_FREQ;
             break;

        case 0x02:                                              /* Reserved                                                 */
             cpu_freq = 0;
             break;

        case 0x03:                                              /* PLL clock                                                */
             pll_mult = (AT91C_BASE_CKGR->CKGR_PLLR & 0x07FF0000) >> 16;
             pll_div  = (AT91C_BASE_CKGR->CKGR_PLLR & 0x000000FF) >>  0;

             if (pll_div == 0) {
                 cpu_freq = 0;                                  /* If PLL divider is zero, then PLL output is zero.         */
             } else if (pll_mult == 0) {
                 cpu_freq = MAIN_XTAL_FREQ;                     /* If PLL multiplier is zero, then PLL is disabled.         */
             } else {
                 cpu_freq = (MAIN_XTAL_FREQ / pll_div) * (pll_mult + 1);
             }
             break;
    }
                                                                /* ---------------- MASTER CLOCK PRESCALER ---------------- */
                                                                /* Read the Master Clock divider                            */
    mclk_div = (AT91C_BASE_PMC->PMC_MCKR >> 2) & 0x07;

    mclk_div = 1 << mclk_div;                                   /* Convert 0-7 into 1, 2, 4, 8, 16, 32, or 64               */

    if (mclk_div >= 128) {                                      /* Divider pattern for 128 is reserved                      */
        return (cpu_freq);
    }

    cpu_freq = cpu_freq / mclk_div;

    return (cpu_freq);
}

/*
*********************************************************************************************************
*                                             Ext_SRAM_Init()
*
* Description : Initialize the External SRAM.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s)
*   (1) This function SHOULD be called before access external SRAM
*   (2) Do not use external functions such as printf(), since this function is called in cstartup.S
*
*********************************************************************************************************
*/

void Ext_SRAM_Init(void)
{

	//-----------------------
	// Clock enable
	//-----------------------
	AT91C_BASE_PMC->PMC_PCER = ((1 << AT91C_ID_PIOA) | (1 << AT91C_ID_PIOB) | (1 << AT91C_ID_PIOC)); //clock Enable

	//-----------------------
	// PIO disable
	//-----------------------
	// PIOA : NBS1
	AT91C_BASE_PIOA->PIO_PDR = ETK_NBS1;
	// PIOB : NBS0(A0), A[1..17]
	AT91C_BASE_PIOB->PIO_PDR = ETK_EBI_A00A17;
	// PIOC : D[0..15], A[18..22], NWE, NRD, NCS0
	AT91C_BASE_PIOC->PIO_PDR = (EXT_EBI_D0D15 | ETK_EBI_A18A22 | ETK_NWE | ETK_NRD | ETK_NCS0);

	//-----------------------
	// Peripheral Select
	//-----------------------
	// PIOA : NBS1
	AT91C_BASE_PIOA->PIO_BSR = ETK_NBS1;
	// PIOB : NBS0(A0), A[1..17]
	AT91C_BASE_PIOB->PIO_BSR = ETK_EBI_A00A17;
	// PIOC : D[0..15], A[18..22]
	AT91C_BASE_PIOC->PIO_ASR = (EXT_EBI_D0D15 | ETK_EBI_A18A22);
	// PIOC : NWE, NRD, NCS0
	AT91C_BASE_PIOC->PIO_BSR = ETK_NWE | ETK_NRD | ETK_NCS0;

	//-----------------------
	// Wait state 
	//-----------------------
	// 3 wait states, Wait state enable, 0 data float time, 16bit wide device, 16bit wide bus
	AT91C_BASE_SMC->SMC2_CSR[0] = (SMC_CSR_NWS_ETK | SMC_CSR_WSEN_ETK | SMC_CSR_TDF_ETK | SMC_CSR_BAT_ETK | SMC_CSR_DBW_ETK);
	
}

/*
*********************************************************************************************************
*                                             Ext_SRAM_Clean()
*
* Description : Clear the External SRAM.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s)     : (1) This function SHOULD be called before access external SRAM
*********************************************************************************************************
*/

void Ext_SRAM_Clean(void)
{
	INT16U *ptr, *ptr_end;

	ptr = (INT16U *)ext_sram_start;
	ptr_end = (INT16U *)(ext_sram_size + ext_sram_start);

	for ( ; ptr_end != ptr; ) 		*ptr++ = ESRAM_INIT_PAT;
}


/*
*********************************************************************************************************
*                                             SevenSeg_Init()
*
* Description : Init. the seven segment interface.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s) 
*
*********************************************************************************************************
*/

void SevenSeg_Init()
{
	AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_PIOB); //clock Ensble
	
	AT91C_BASE_PIOB->PIO_PER  = (ETK_SHIFT_DATA|ETK_SHIFT_CLOCK|ETK_7SEG_LATCH); 	//PIO  
	AT91C_BASE_PIOB->PIO_OER  = (ETK_SHIFT_DATA|ETK_SHIFT_CLOCK|ETK_7SEG_LATCH);	    //PIO   
}



/*
*********************************************************************************************************
*                                             INT8U Conv_To_7seg(INT8U c)
*
* Description : Convert from hexadecimal to code of seven segment.
*
* Argument(s) : c    hexadecimal
*
* Return(s)   :         seven segment code
*
* Note(s) 
*    (1). The DBof7Seg[] is defined in the etk.h
*
*********************************************************************************************************
*/

static INT8U Conv_To_7seg(INT8U c)
{
	c = c & 0x0f; 		//LNIBBLE;

	return(DBof7Seg[(INT8U)c]);
}


/*
*********************************************************************************************************
*                                             SevenSeg_Putui(INT8U ui)
*
* Description : Transmit the code of seven segment to serial buffer(74hc595) to display
*
* Argument(s) : ui     code of seven segment to display
*
* Return(s)   : none.
*
* Note(s) 
*
*********************************************************************************************************
*/


void SevenSeg_Putui(INT8U ui)
{
	INT8U c, i, t;

    c = Conv_To_7seg(ui);								// convert digit to seven segment code.

	for (i=8;i;i--) 
	{
		t = c;
		t &= 0x80;
		if(t)											// data out
			AT91C_BASE_PIOB->PIO_SODR = ETK_SHIFT_DATA;
		else
			AT91C_BASE_PIOB->PIO_CODR = ETK_SHIFT_DATA;
			
		AT91C_BASE_PIOB->PIO_SODR = ETK_SHIFT_CLOCK;		// shift clock out
		AT91C_BASE_PIOB->PIO_CODR = ETK_SHIFT_CLOCK;
		c <<= 1;										// shift 1 bit	
	}
	
	AT91C_BASE_PIOB->PIO_SODR = ETK_7SEG_LATCH;			// latch clock out
	AT91C_BASE_PIOB->PIO_CODR = ETK_7SEG_LATCH;

}

/* ++ ji(Rev 1.1)
*********************************************************************************************************
*                                             console_irq_handler(void)
*
* Description : UART0 Rx ͷƮ ڵ鷯
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s) 		  UART0 شϴ ڵ鷯.
*								rxq_buff ȯť Ͽ Rx  ͷƮ ߻ ȯť ͸ 
*								rxq_buff etk.h  Ǿ 
*
*********************************************************************************************************
*/
void console_irq_handler(void)
{
	CPU_INT08U c;
	
	c = (CPU_INT08U)(AT91C_BASE_US0->US_RHR & 0x00FF);			// Rx ۿ ͸ о
	rxq_buff[rxq_head] = c;																	// ȯ ť  
	rxq_head++;
	rxq_head &= RX_QUEUE_MASK;
	
}

/* ++ ji(Rev 1.1)
*********************************************************************************************************
*                  irq_request ( CPU_INT32U irq, CPU_INT32U type, void (*irq_func ) (void ) )
*
* Description : ͷƮ  Լ
*
* Argument(s) : irq					ͷƮ Enable ġ ȣ
*								type				ͷƮ  Ÿ
*								(*irq_func)	ͷƮ ڵ鷯(ISR) 
*
* Return(s)   : none.
*
* Note(s) 		  ͷƮ Enable, ڵ鷯(ISR), 켱, ͷƮ  Ÿ 
*
*********************************************************************************************************
*/
void irq_request ( CPU_INT32U irq, CPU_INT32U type, void (*irq_func ) (void ) )
{
	CPU_INT32U mask;
	mask = 0x1 << irq ;
	AT91C_BASE_AIC->AIC_IECR = mask;													// ͷƮ Enable  
	AT91C_BASE_AIC->AIC_SVR[irq] = (CPU_INT32U) irq_func;			// ͷƮ  ƾ 
	if(irq == 1)	AT91C_BASE_AIC->AIC_SMR[irq] = type | 7;		// 켱  ͷƮ Ÿ
	else					AT91C_BASE_AIC->AIC_SMR[irq] = type | AT91C_AIC_PRIOR_HIGHEST  ;		// 켱  ͷƮ Ÿ
}

/* ++ ji(Rev 1.1)
*********************************************************************************************************
*                  											Serial_Init(void)
*
* Description : UART0 ʱȭ Լ
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s) 		  UART0  ϱ  
*
*********************************************************************************************************
*/
void  Serial_Init(void)
{
    CPU_INT32U  pclk_freq;

    pclk_freq                  = BSP_CPU_ClkFreq1();            /* Get the UART input clock frequency                       */

    AT91C_BASE_PIOA->PIO_PDR   = ETK_RXD0 | ETK_TXD0;       		/* Set GPIOA pins 5 and 6 as UART0 pins                */
    AT91C_BASE_PIOA->PIO_ASR   = ETK_RXD0 | ETK_TXD0;       		/* Select GPIOA attached peripheral (UART0)                  */

    AT91C_BASE_US0->US_IER = AT91C_US_RXRDY;                      
                                                
    AT91C_BASE_US0->US_CR    = AT91C_US_RXEN                  	/* Enable the receiver                                      */
                               | AT91C_US_TXEN|AT91C_US_RTSDIS |AT91C_US_DTRDIS;                 /* Enable the transmitter                                   */
    AT91C_BASE_US0->US_MR = 0; 																	// Set Default
    AT91C_BASE_US0->US_MR   = AT91C_US_USMODE_NORMAL         		/* Normal mode selected                                     */
                               | AT91C_US_PAR_NONE|AT91C_US_CHRL_8_BITS|AT91C_US_NBSTOP_1_BIT ;  /* No parity bit selected                                   */

                                                                /* Set the UART0 baud rate to 115,200                        */
    AT91C_BASE_US0->US_BRGR = (CPU_INT16U)((pclk_freq) / 115200 / 16);

    AT91C_BASE_PMC->PMC_PCER  = (1 << AT91C_ID_US0);           	/* Enable the UART0 peripheral clock */
		AT91C_BASE_PMC->PMC_IER = 1 << AT91C_PMC_MCKRDY;							// UART0 ͷƮ  Ŭ 
		
    irq_request( AT91C_ID_US0, AT91C_AIC_SRCTYPE_HIGH_LEVEL, console_irq_handler ); // ͷƮ ڵ鷯 
   
}

/* ++ ji(Rev 1.1)
*********************************************************************************************************
*                  											Serial_dis(void)
*
* Description : ͷƮ Disable
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s) 		  UART0 ͷƮ disable, UART0ġ ѵ Լ
*
*********************************************************************************************************
*/
void Serial_dis(void)
{
	AT91C_BASE_US0->US_IDR = AT91C_US_RXRDY;
	AT91C_BASE_AIC->AIC_IDCR = (1 << AT91C_ID_US0);
	AT91C_BASE_PMC->PMC_PCDR  = (1 << AT91C_ID_US0);
	AT91C_BASE_PMC->PMC_IDR = 1 << AT91C_PMC_MCKRDY;
	
}

/* ++ ji(Rev 1.1)
*********************************************************************************************************
*                  									Serial_WrByte (CPU_INT08U tx_byte)
*
* Description :  ڸ ۽
*
* Argument(s) : tx_byte		  		
*
* Return(s)   : none.
*
* Note(s) 		  
*
*********************************************************************************************************
*/
void  Serial_WrByte (CPU_INT08U tx_byte)
{
    while ((AT91C_BASE_US0-> US_CSR & AT91C_US_TXRDY) == 0){  /*  Wait for room in the transmit register.                 */
        ;
    }
    AT91C_BASE_US0 ->US_THR= tx_byte;
    
}

/* ++ ji(Rev 1.1)
*********************************************************************************************************
*                  									Serial_WrStr (CPU_CHAR *tx_str)
*
* Description : ڿ ۽
*
* Argument(s) : tx_str	 ڿ ּ
*
* Return(s)   : none.
*
* Note(s) 		  
*
*********************************************************************************************************
*/
void  Serial_WrStr (CPU_CHAR *tx_str)
{
    while ((*tx_str) != 0) {
        Serial_WrByte(*tx_str++);
    }
}

/* ++ ji(Rev 1.1)
*********************************************************************************************************
*                  									Serial_RdByte (void)
*
* Description : ѹڸ 
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s) 		   ͷƮ Լ
*
*********************************************************************************************************
*/
CPU_INT08U Serial_RdByte (void)
{
    CPU_INT08U      rx_byte;

    while ((AT91C_BASE_US0->US_CSR & AT91C_US_RXRDY) == 0) {     /*  Wait for a byte to show up.                         */
        OSTimeDly(2);
    }
    rx_byte = (CPU_INT08U)(AT91C_BASE_US0->US_RHR & 0x00FF);     /* Read the character.                                  */
    return (rx_byte);
}

/* ++ ji(Rev 1.1)
*********************************************************************************************************
*                  					Serial_RdStr (CPU_CHAR *rx_str, CPU_INT32U len)
*
* Description : ڿ 
*
* Argument(s) : rx_str	ť  
*								len			ڿ 
*
* Return(s)   : none.
*
* Note(s) 		  Rx ͷƮ ڵ鷯  ť  ͸ о
*
*********************************************************************************************************
*/
void  Serial_RdStr (CPU_CHAR *rx_str, CPU_INT32U len)
{
	static CPU_INT32U  count = 0;

  while (count < len ) {
       Serial_Printf((CPU_CHAR *)"%c", rx_str[count]);
          count++;  
  }
	count = 0;	
}

/* ++ ji(Rev 1.1)
*********************************************************************************************************
*                  					Serial_Printf (CPU_CHAR *format, ...)
*
* Description : ڿ UART0  ̿Ͽ ۽
*
* Argument(s) : *format	 ڿ 
*
* Return(s)   : none.
*
* Note(s) 		  信 ° ڿ ѵ Serial_WrStr() 
*
*********************************************************************************************************
*/
void  Serial_Printf (CPU_CHAR *format, ...)
{
    static  CPU_CHAR  buffer[80 + 1];
    va_list   vArgs;

    va_start(vArgs, format);
		vsprintf((char *)buffer, (char const *)format, vArgs);
		va_end(vArgs);

    Serial_WrStr((CPU_CHAR*) buffer);
}
/* ++ ji(Rev 1.2)
*********************************************************************************************************
*                  											Serial_Init(void)
*
* Description : PWM ʱȭ Լ
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Note(s) 		  PWM ϱ  
*
*********************************************************************************************************
*/
void PWM_Init()
{
	CPU_INT32U  pclk_freq;
    pclk_freq = BSP_CPU_ClkFreq1(); 
	AT91C_BASE_PIOA->PIO_ASR  = ETK_PWM2;  // Select A Register	
	AT91C_BASE_PIOA->PIO_PDR = ETK_PWM2;//Output Enable Register
	AT91C_BASE_PMC->PMC_PCER  = (1 << AT91C_ID_PWMC); //Peripheral Clock Enable Register
	AT91C_BASE_PWMC->PWMC_MR = 0x00000801;//ֿ ٽ  ϴ Ѿ.
	AT91C_BASE_PWMC->PWMC_ENA = AT91C_PWMC_CHID2;//PWM Enable Register
	AT91C_BASE_PWMC->PWMC_IDR = AT91C_PWMC_CHID2;//PWM Interrupt Disable Register
	AT91C_BASE_PWMC_CH2->PWMC_CMR = 0x00000308;//PWM Channel Mode Register
	AT91C_BASE_PWMC_CH2->PWMC_CPRDR = (pclk_freq/(2*256));//PWM Channel Period Register
}




